@@ -20,7 +20,7 @@
 #define PHP_XDEBUG_H
 
 #define XDEBUG_NAME       "Xdebug"
-#define XDEBUG_VERSION    "2.1.2"
+#define XDEBUG_VERSION    "2.2.1"
 #define XDEBUG_AUTHOR     "Derick Rethans"
 #define XDEBUG_COPYRIGHT  "Copyright (c) 2002-2012 by Derick Rethans"
 #define XDEBUG_COPYRIGHT_SHORT "Copyright (c) 2002-2012"
@@ -32,12 +32,17 @@
 #include "xdebug_handlers.h"
 #include "xdebug_hash.h"
 #include "xdebug_llist.h"
+#include "xdebug_code_coverage.h"
 
 extern zend_module_entry xdebug_module_entry;
 #define phpext_xdebug_ptr &xdebug_module_entry
 
 #define MICRO_IN_SEC 1000000.00
 
+#define OUTPUT_NOT_CHECKED -1
+#define OUTPUT_IS_TTY       1
+#define OUTPUT_NOT_TTY      0
+
 #ifdef PHP_WIN32
 #define PHP_XDEBUG_API __declspec(dllexport)
 #else
@@ -81,6 +86,10 @@ PHP_RSHUTDOWN_FUNCTION(xdebug);
 PHP_MINFO_FUNCTION(xdebug);
 ZEND_MODULE_POST_ZEND_DEACTIVATE_D(xdebug);
 
+#ifndef PHP_WIN32
+int xdebug_is_output_tty();
+#endif
+
 /* call stack functions */
 PHP_FUNCTION(xdebug_get_stack_depth);
 PHP_FUNCTION(xdebug_get_function_stack);
@@ -147,7 +156,6 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	zend_bool     show_ex_trace;
 	zend_bool     show_local_vars;
 	zend_bool     show_mem_delta;
-	char         *manual_url;
 	double        start_time;
 	HashTable    *active_symbol_table;
 	zend_execute_data *active_execute_data;
@@ -165,6 +173,7 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	FILE         *trace_file;
 	zend_bool     do_trace;
 	zend_bool     auto_trace;
+	zend_bool     trace_enable_trigger;
 	char         *trace_output_dir;
 	char         *trace_output_name;
 	long          trace_options;
@@ -178,13 +187,19 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	long          display_max_data;
 	long          display_max_depth;
 
+	zend_bool     cli_color;
+	int           output_is_tty;
+
 	/* used for code coverage */
+	zend_bool     coverage_enable;
 	zend_bool     do_code_coverage;
 	xdebug_hash  *code_coverage;
 	zend_bool     code_coverage_unused;
 	zend_bool     code_coverage_dead_code_analysis;
 	unsigned int  function_count;
 	int           reserved_offset;
+	char                 *previous_filename;
+	xdebug_coverage_file *previous_file;
 
 	/* used for collection errors */
 	zend_bool     do_collect_errors;
@@ -219,7 +234,8 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	FILE         *remote_log_file;  /* File handler for protocol log */
 	long          remote_cookie_expire_time; /* Expire time for the remote-session cookie */
 
-	char         *ide_key;    /* from environment, USER, USERNAME or empty */
+	char         *ide_key; /* As Xdebug uses it, from environment, USER, USERNAME or empty */
+	char         *ide_key_setting; /* Set through php.ini and friends */
 
 	/* remote debugging globals */
 	zend_bool     remote_enabled;
@@ -245,10 +261,7 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	char         *lasttransid;
 
 	/* output redirection */
-	php_output_globals stdio;
-	int stdout_redirected;
-	int stderr_redirected;
-	int stdin_redirected;
+	int           stdout_mode;
 
 	/* aggregate profiling */
 	HashTable  aggr_calls;
@@ -256,6 +269,10 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 
 	/* scream */
 	zend_bool  do_scream;
+	zend_bool  in_at;
+
+	/* in-execution checking */
+	zend_bool  in_execution;
 ZEND_END_MODULE_GLOBALS(xdebug)
 
 #ifdef ZTS
