@@ -41,9 +41,16 @@
 #include "main/php_ini.h"
 
 #ifndef NAME_MAX
-# define NAME_MAX (MAXNAMELEN-1)
+# ifdef _AIX
+#  include <unistd.h>
+#  define NAME_MAX pathconf("/dev/null",_PC_NAME_MAX)
+# else
+#  define NAME_MAX (MAXNAMELEN-1)
+# endif
 #endif
 
+ZEND_EXTERN_MODULE_GLOBALS(xdebug)
+
 #define READ_BUFFER_SIZE 128
 
 char* xdebug_fd_read_line_delim(int socket, fd_buf *context, int type, unsigned char delim, int *length)
@@ -312,6 +319,12 @@ char *xdebug_path_from_url(const char *fileurl TSRMLS_DC)
 	return ret;
 }
 
+#if PHP_VERSION_ID >= 50400
+# define VIRTUAL_FILE_EX_TSRMLS_CC TSRMLS_CC
+#else
+# define VIRTUAL_FILE_EX_TSRMLS_CC
+#endif
+
 /* fake URI's per IETF RFC 1738 and 2396 format */
 char *xdebug_path_to_url(const char *fileurl TSRMLS_DC)
 {
@@ -339,7 +352,7 @@ char *xdebug_path_to_url(const char *fileurl TSRMLS_DC)
 		new_state.cwd = strdup(cwd);
 		new_state.cwd_length = strlen(cwd);
 
-		if(!virtual_file_ex(&new_state, fileurl, NULL, 1)) {
+		if (!virtual_file_ex(&new_state, fileurl, NULL, 1 VIRTUAL_FILE_EX_TSRMLS_CC)) {
 			char *s = estrndup(new_state.cwd, new_state.cwd_length);
 			tmp = xdebug_sprintf("file://%s",s);
 			efree(s);
@@ -529,8 +542,9 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 			switch (*format)
 			{
 				case 'c': /* crc32 of the current working directory */
-					VCWD_GETCWD(cwd, 127);
-					xdebug_str_add(&fname, xdebug_sprintf("%lu", xdebug_crc32(cwd, strlen(cwd))), 1);
+					if (VCWD_GETCWD(cwd, 127)) {
+						xdebug_str_add(&fname, xdebug_sprintf("%lu", xdebug_crc32(cwd, strlen(cwd))), 1);
+					}
 					break;
 
 				case 'p': /* pid */
@@ -584,10 +598,11 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 				}	break;
 
 				case 'H':   /* $_SERVER['HTTP_HOST'] */
+				case 'U':   /* $_SERVER['UNIQUE_ID'] */
 				case 'R': { /* $_SERVER['REQUEST_URI'] */
 					zval **data;
 					char *char_ptr, *strval;
-					int retval;
+					int retval = FAILURE;
 
 					if (PG(http_globals)[TRACK_VARS_SERVER]) {
 						switch (*format) {
@@ -597,6 +612,9 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 						case 'R':
 							retval = zend_hash_find(Z_ARRVAL_P(PG(http_globals)[TRACK_VARS_SERVER]), "REQUEST_URI", sizeof("REQUEST_URI"), (void **) &data);
 							break;
+						case 'U':
+							retval = zend_hash_find(Z_ARRVAL_P(PG(http_globals)[TRACK_VARS_SERVER]), "UNIQUE_ID", sizeof("UNIQUE_ID"), (void **) &data);
+							break;
 						}
 
 						if (retval == SUCCESS) {
@@ -649,3 +667,30 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 
 	return fname.l;
 }
+
+void xdebug_open_log(TSRMLS_D)
+{
+	/* initialize remote log file */
+	XG(remote_log_file) = NULL;
+	if (XG(remote_log) && strlen(XG(remote_log))) {
+		XG(remote_log_file) = xdebug_fopen(XG(remote_log), "a", NULL, NULL);
+	}
+	if (XG(remote_log_file)) {
+		char *timestr = xdebug_get_time();
+		fprintf(XG(remote_log_file), "Log opened at %s\n", timestr);
+		fflush(XG(remote_log_file));
+		xdfree(timestr);
+	}
+}
+
+void xdebug_close_log(TSRMLS_D)
+{
+	if (XG(remote_log_file)) {
+		char *timestr = xdebug_get_time();
+		fprintf(XG(remote_log_file), "Log closed at %s\n\n", timestr);
+		fflush(XG(remote_log_file));
+		xdfree(timestr);
+		fclose(XG(remote_log_file));
+		XG(remote_log_file) = NULL;
+	}
+}
