@@ -61,6 +61,8 @@ char *xdebug_path_to_url(const char *fileurl TSRMLS_DC);
 char *xdebug_path_from_url(const char *fileurl TSRMLS_DC);
 FILE *xdebug_fopen(char *fname, char *mode, char *extension, char **new_fname);
 int xdebug_format_output_filename(char **filename, char *format, char *script_name);
+void xdebug_open_log(TSRMLS_D);
+void xdebug_close_log(TSRMLS_D);
 
 #define XDEBUG_CRC32(crc, ch)	 (crc = (crc >> 8) ^ xdebug_crc32tab[(crc ^ (ch)) & 0xff])
 
