@@ -44,6 +44,7 @@
 #include "ext/standard/info.h"
 #include "ext/standard/php_string.h"
 #include "php_globals.h"
+#include "main/php_output.h"
 #include "ext/standard/php_var.h"
 
 
@@ -76,6 +77,7 @@ void (*xdebug_new_error_cb)(int type, const char *error_filename, const uint err
 void xdebug_error_cb(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args);
 
 static int xdebug_header_handler(sapi_header_struct *h XG_SAPI_HEADER_OP_DC, sapi_headers_struct *s TSRMLS_DC);
+static int xdebug_ub_write(const char *string, unsigned int lenght TSRMLS_DC);
 
 static void xdebug_throw_exception_hook(zval *exception TSRMLS_DC);
 int xdebug_exit_handler(ZEND_OPCODE_HANDLER_ARGS);
@@ -84,6 +86,9 @@ int zend_xdebug_initialised = 0;
 int zend_xdebug_global_offset = -1;
 
 int (*xdebug_orig_header_handler)(sapi_header_struct *h XG_SAPI_HEADER_OP_DC, sapi_headers_struct *s TSRMLS_DC);
+int (*xdebug_orig_ub_write)(const char *string, unsigned int len TSRMLS_DC);
+
+static int xdebug_trigger_enabled(int setting, char *var_name TSRMLS_DC);
 
 zend_function_entry xdebug_functions[] = {
 	PHP_FE(xdebug_get_stack_depth,       NULL)
@@ -140,7 +145,7 @@ zend_module_entry xdebug_module_entry = {
 	PHP_MINIT(xdebug),
 	PHP_MSHUTDOWN(xdebug),
 	PHP_RINIT(xdebug),
-	NULL,
+	PHP_RSHUTDOWN(xdebug),
 	PHP_MINFO(xdebug),
 	XDEBUG_VERSION,
 #if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 2) || PHP_MAJOR_VERSION >= 6
@@ -197,19 +202,6 @@ static PHP_INI_MH(OnUpdateSession)
 	DUMP_TOK(session);
 }
 
-static PHP_INI_MH(OnUpdateIDEKey)
-{
-	if (XG(ide_key)) {
-		xdfree(XG(ide_key));
-	}
-	if (!new_value) {
-		XG(ide_key) = NULL;
-	} else {
-		XG(ide_key) = xdstrdup(new_value);
-	}
-	return SUCCESS;
-}
-
 static PHP_INI_MH(OnUpdateDebugMode)
 {
 	if (!new_value) {
@@ -236,10 +228,12 @@ static PHP_INI_MH(OnUpdateDebugMode)
 PHP_INI_BEGIN()
 	/* Debugger settings */
 	STD_PHP_INI_BOOLEAN("xdebug.auto_trace",      "0",                  PHP_INI_ALL,    OnUpdateBool,   auto_trace,        zend_xdebug_globals, xdebug_globals)
+	STD_PHP_INI_BOOLEAN("xdebug.trace_enable_trigger", "0",             PHP_INI_SYSTEM|PHP_INI_PERDIR, OnUpdateBool,   trace_enable_trigger, zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.trace_output_dir",  XDEBUG_TEMP_DIR,      PHP_INI_ALL,    OnUpdateString, trace_output_dir,  zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.trace_output_name", "trace.%c",           PHP_INI_ALL,    OnUpdateString, trace_output_name, zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.trace_format",      "0",                  PHP_INI_ALL,    OnUpdateLong,   trace_format,      zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.trace_options",     "0",                  PHP_INI_ALL,    OnUpdateLong,   trace_options,     zend_xdebug_globals, xdebug_globals)
+	STD_PHP_INI_BOOLEAN("xdebug.coverage_enable", "1",                  PHP_INI_SYSTEM, OnUpdateBool,   coverage_enable,   zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_BOOLEAN("xdebug.collect_includes","1",                  PHP_INI_ALL,    OnUpdateBool,   collect_includes,  zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.collect_params",  "0",                    PHP_INI_ALL,    OnUpdateLong,   collect_params,    zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_BOOLEAN("xdebug.collect_return",  "0",                  PHP_INI_ALL,    OnUpdateBool,   collect_return,    zend_xdebug_globals, xdebug_globals)
@@ -248,7 +242,6 @@ PHP_INI_BEGIN()
 	STD_PHP_INI_BOOLEAN("xdebug.default_enable",  "1",                  PHP_INI_ALL,    OnUpdateBool,   default_enable,    zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_BOOLEAN("xdebug.extended_info",   "1",                  PHP_INI_SYSTEM, OnUpdateBool,   extended_info,     zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.file_link_format",  "",                   PHP_INI_ALL,    OnUpdateString, file_link_format,  zend_xdebug_globals, xdebug_globals)
-	STD_PHP_INI_ENTRY("xdebug.manual_url",        "http://www.php.net", PHP_INI_ALL,    OnUpdateString, manual_url,        zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.max_nesting_level", "100",                PHP_INI_ALL,    OnUpdateLong,   max_nesting_level, zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_BOOLEAN("xdebug.overload_var_dump", "1", PHP_INI_SYSTEM|PHP_INI_PERDIR, OnUpdateBool,   overload_var_dump, zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_BOOLEAN("xdebug.show_exception_trace",  "0",            PHP_INI_ALL,    OnUpdateBool,   show_ex_trace,     zend_xdebug_globals, xdebug_globals)
@@ -285,13 +278,14 @@ PHP_INI_BEGIN()
 	STD_PHP_INI_BOOLEAN("xdebug.remote_autostart","0",                  PHP_INI_ALL,    OnUpdateBool,   remote_autostart,  zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_BOOLEAN("xdebug.remote_connect_back","0",               PHP_INI_ALL,    OnUpdateBool,   remote_connect_back,  zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.remote_log",        "",                   PHP_INI_ALL,    OnUpdateString, remote_log,        zend_xdebug_globals, xdebug_globals)
-	PHP_INI_ENTRY("xdebug.idekey",                "",                   PHP_INI_ALL,    OnUpdateIDEKey)
+	STD_PHP_INI_ENTRY("xdebug.idekey",            "",                   PHP_INI_ALL,    OnUpdateString, ide_key_setting,   zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.remote_cookie_expire_time", "3600",       PHP_INI_ALL,    OnUpdateLong,   remote_cookie_expire_time, zend_xdebug_globals, xdebug_globals)
 
 	/* Variable display settings */
 	STD_PHP_INI_ENTRY("xdebug.var_display_max_children", "128",         PHP_INI_ALL,    OnUpdateLong,   display_max_children, zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.var_display_max_data",     "512",         PHP_INI_ALL,    OnUpdateLong,   display_max_data,     zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.var_display_max_depth",    "3",           PHP_INI_ALL,    OnUpdateLong,   display_max_depth,    zend_xdebug_globals, xdebug_globals)
+	STD_PHP_INI_ENTRY("xdebug.cli_color",                "0",           PHP_INI_ALL,    OnUpdateBool,   cli_color,            zend_xdebug_globals, xdebug_globals)
 
 	/* Scream support */
 	STD_PHP_INI_BOOLEAN("xdebug.scream",                 "0",           PHP_INI_ALL,    OnUpdateBool,   do_scream,            zend_xdebug_globals, xdebug_globals)
@@ -299,15 +293,19 @@ PHP_INI_END()
 
 static void php_xdebug_init_globals (zend_xdebug_globals *xg TSRMLS_DC)
 {
-	zend_extension dummy_ext;
-
 	xg->stack                = NULL;
 	xg->level                = 0;
 	xg->do_trace             = 0;
 	xg->trace_file           = NULL;
+	xg->coverage_enable      = 0;
+	xg->previous_filename    = "";
+	xg->previous_file        = NULL;
 	xg->do_code_coverage     = 0;
 	xg->breakpoint_count     = 0;
 	xg->ide_key              = NULL;
+	xg->output_is_tty        = OUTPUT_NOT_CHECKED;
+	xg->stdout_mode          = 0;
+	xg->in_at                = 0;
 
 	xdebug_llist_init(&xg->server, xdebug_superglobals_dump_dtor);
 	xdebug_llist_init(&xg->get, xdebug_superglobals_dump_dtor);
@@ -327,6 +325,12 @@ static void php_xdebug_init_globals (zend_xdebug_globals *xg TSRMLS_DC)
 		sapi_module.header_handler = xdebug_header_handler;
 	}
 	xg->headers = NULL;
+
+	/* Capturing output */
+	if (sapi_module.ub_write != xdebug_ub_write) {
+		xdebug_orig_ub_write = sapi_module.ub_write;
+		sapi_module.ub_write = xdebug_ub_write;
+	}
 }
 
 static void php_xdebug_shutdown_globals (zend_xdebug_globals *xg TSRMLS_DC)
@@ -341,21 +345,34 @@ static void php_xdebug_shutdown_globals (zend_xdebug_globals *xg TSRMLS_DC)
 	xdebug_llist_empty(&xg->session, NULL);
 }
 
-void xdebug_env_key()
+char *xdebug_env_key(TSRMLS_D)
 {
-	char *ide_key = getenv("DBGP_IDEKEY");
-	if (!ide_key || !*ide_key) {
-		ide_key = getenv("USER");
-		if (!ide_key || !*ide_key) {
-			ide_key = getenv("USERNAME");
-		}
-	}
+	char *ide_key;
+
+	ide_key = XG(ide_key_setting);
 	if (ide_key && *ide_key) {
-		zend_alter_ini_entry("xdebug.idekey", sizeof("xdebug.idekey"), ide_key, strlen(ide_key), PHP_INI_SYSTEM, PHP_INI_STAGE_ACTIVATE);
+		return ide_key;
 	}
+
+	ide_key = getenv("DBGP_IDEKEY");
+	if (ide_key && *ide_key) {
+		return ide_key;
+	}
+
+	ide_key = getenv("USER");
+	if (ide_key && *ide_key) {
+		return ide_key;
+	}
+
+	ide_key = getenv("USERNAME");
+	if (ide_key && *ide_key) {
+		return ide_key;
+	}
+
+	return NULL;
 }
 
-void xdebug_env_config()
+void xdebug_env_config(TSRMLS_D)
 {
 	char       *config = getenv("XDEBUG_CONFIG");
 	xdebug_arg *parts;
@@ -364,7 +381,6 @@ void xdebug_env_config()
 		XDEBUG_CONFIG format:
 		XDEBUG_CONFIG=var=val var=val
 	*/
-	xdebug_env_key();
 	if (!config) {
 		return;
 	}
@@ -403,7 +419,10 @@ void xdebug_env_config()
 			name = "xdebug.remote_mode";
 		} else
 		if (strcasecmp(envvar, "idekey") == 0) {
-			name = "xdebug.idekey";
+			if (XG(ide_key)) {
+				xdfree(XG(ide_key));
+			}
+			XG(ide_key) = xdstrdup(envval);
 		} else
 		if (strcasecmp(envvar, "profiler_enable") == 0) {
 			name = "xdebug.profiler_enable";
@@ -423,6 +442,9 @@ void xdebug_env_config()
 		if (strcasecmp(envvar, "remote_cookie_expire_time") == 0) {
 			name = "xdebug.remote_cookie_expire_time";
 		}
+		else if (strcasecmp(envvar, "cli_color") == 0) {
+			name = "xdebug.cli_color";
+		}
 
 		if (name) {
 			zend_alter_ini_entry(name, strlen(name) + 1, envval, strlen(envval), PHP_INI_SYSTEM, PHP_INI_STAGE_ACTIVATE);
@@ -434,24 +456,34 @@ void xdebug_env_config()
 
 static int xdebug_silence_handler(ZEND_OPCODE_HANDLER_ARGS)
 {
-    if (XG(do_scream)) {
-        execute_data->opline++;
-        return ZEND_USER_OPCODE_CONTINUE;
-    }
-    return ZEND_USER_OPCODE_DISPATCH;
+	zend_op *cur_opcode = *EG(opline_ptr);
+
+	if (XG(do_scream)) {
+		execute_data->opline++;
+		if (cur_opcode->opcode == ZEND_BEGIN_SILENCE) {
+			XG(in_at) = 1;
+		} else {
+			XG(in_at) = 0;
+		}
+		return ZEND_USER_OPCODE_CONTINUE;
+	}
+	return ZEND_USER_OPCODE_DISPATCH;
 }
 
 static int xdebug_include_or_eval_handler(ZEND_OPCODE_HANDLER_ARGS)
 {
 	zend_op *opline = execute_data->opline;
 
+#if PHP_VERSION_ID >= 50399
+	if (opline->extended_value == ZEND_EVAL) {
+#else
 	if (Z_LVAL(opline->op2.u.constant) == ZEND_EVAL) {
+#endif
 		zval *inc_filename;
 		zval tmp_inc_filename;
 		int  is_var;
-		int  tmp_len;
 
-		inc_filename = xdebug_get_zval(execute_data, &opline->op1, execute_data->Ts, &is_var);
+		inc_filename = xdebug_get_zval(execute_data, opline->XDEBUG_TYPE(op1), &opline->op1, execute_data->Ts, &is_var);
 		
 		/* If there is no inc_filename, we're just bailing out instead */
 		if (!inc_filename) {
@@ -478,6 +510,42 @@ static int xdebug_include_or_eval_handler(ZEND_OPCODE_HANDLER_ARGS)
 	return ZEND_USER_OPCODE_DISPATCH;
 }
 
+int xdebug_is_output_tty(TSRMLS_D)
+{
+	if (XG(output_is_tty) == OUTPUT_NOT_CHECKED) {
+#ifndef PHP_WIN32
+		XG(output_is_tty) = isatty(STDOUT_FILENO);
+#else
+		XG(output_is_tty) = getenv("ANSICON");
+#endif
+	}
+	return (XG(output_is_tty));
+}
+
+#if 0
+int static xdebug_stack_insert_top(zend_stack *stack, const void *element, int size)
+{
+	int i;
+
+    if (stack->top >= stack->max) {    /* we need to allocate more memory */
+        stack->elements = (void **) erealloc(stack->elements,
+                   (sizeof(void **) * (stack->max += 64)));
+        if (!stack->elements) {
+            return FAILURE;
+        }
+    }
+
+	/* move all existing ones up */
+	for (i = stack->top; i >= 0; i--) {
+		stack->elements[i + 1] = stack->elements[i];
+	}
+
+	/* replace top handler */
+    stack->elements[0] = (void *) emalloc(size);
+    memcpy(stack->elements[0], element, size);
+    return stack->top++;
+}
+#endif
 
 PHP_MINIT_FUNCTION(xdebug)
 {
@@ -509,56 +577,62 @@ PHP_MINIT_FUNCTION(xdebug)
 	/* Overload the "exit" opcode */
 	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(exit, ZEND_EXIT);
 
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_JMP);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_JMPZ);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_JMPNZ);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_JMPZ_EX);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_IDENTICAL);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_NOT_IDENTICAL);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_EQUAL);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_NOT_EQUAL);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_SMALLER);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_SMALLER_OR_EQUAL);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_BOOL_NOT);
+	/* Overload opcodes for code coverage */
+	if (XG(coverage_enable)) {
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_JMP);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_JMPZ);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_JMPZ_EX);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_JMPNZ);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_IDENTICAL);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_NOT_IDENTICAL);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_EQUAL);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_NOT_EQUAL);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_SMALLER);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_SMALLER_OR_EQUAL);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_BOOL_NOT);
 
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ADD);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SUB);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_MUL);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_DIV);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ADD);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SUB);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_MUL);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_DIV);
 
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ADD_ARRAY_ELEMENT);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_RETURN);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_EXT_STMT);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_RAISE_ABSTRACT_ERROR);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SEND_VAR);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SEND_VAR_NO_REF);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SEND_VAL);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_NEW);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_EXT_FCALL_BEGIN);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_CATCH);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_BOOL);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ADD_CHAR);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ADD_STRING);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_INIT_ARRAY);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_DIM_R);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_OBJ_R);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_OBJ_W);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_OBJ_FUNC_ARG);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_DIM_FUNC_ARG);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_DIM_UNSET);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_OBJ_UNSET);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_CLASS);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_CONSTANT);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_CONCAT);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ISSET_ISEMPTY_DIM_OBJ);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_PRE_INC_OBJ);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SWITCH_FREE);
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_QM_ASSIGN);
-#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_DECLARE_LAMBDA_FUNCTION);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ADD_ARRAY_ELEMENT);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_RETURN);
+#if PHP_VERSION_ID >= 50400
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_RETURN_BY_REF);
 #endif
-	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(include_or_eval, ZEND_INCLUDE_OR_EVAL);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_EXT_STMT);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_RAISE_ABSTRACT_ERROR);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SEND_VAR);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SEND_VAR_NO_REF);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SEND_VAL);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_NEW);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_EXT_FCALL_BEGIN);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_CATCH);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_BOOL);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ADD_CHAR);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ADD_STRING);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_INIT_ARRAY);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_DIM_R);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_OBJ_R);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_OBJ_W);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_OBJ_FUNC_ARG);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_DIM_FUNC_ARG);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_DIM_UNSET);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_OBJ_UNSET);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_CLASS);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_CONSTANT);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_CONCAT);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ISSET_ISEMPTY_DIM_OBJ);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_PRE_INC_OBJ);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SWITCH_FREE);
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_QM_ASSIGN);
+#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
+		XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_DECLARE_LAMBDA_FUNCTION);
+#endif
+	}
 
+	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(include_or_eval, ZEND_INCLUDE_OR_EVAL);
 	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(assign, ZEND_ASSIGN);
 	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(assign_add, ZEND_ASSIGN_ADD);
 	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(assign_sub, ZEND_ASSIGN_SUB);
@@ -577,6 +651,10 @@ PHP_MINIT_FUNCTION(xdebug)
 	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(post_inc, ZEND_POST_INC);
 	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(pre_dec, ZEND_PRE_DEC);
 	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(post_dec, ZEND_POST_DEC);
+	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(pre_inc_obj, ZEND_PRE_INC_OBJ);
+	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(post_inc_obj, ZEND_POST_INC_OBJ);
+	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(pre_dec_obj, ZEND_PRE_DEC_OBJ);
+	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(post_dec_obj, ZEND_POST_DEC_OBJ);
 
 	zend_set_user_opcode_handler(ZEND_BEGIN_SILENCE, xdebug_silence_handler);
 	zend_set_user_opcode_handler(ZEND_END_SILENCE, xdebug_silence_handler);
@@ -593,11 +671,18 @@ PHP_MINIT_FUNCTION(xdebug)
 	REGISTER_LONG_CONSTANT("XDEBUG_CC_DEAD_CODE", XDEBUG_CC_OPTION_DEAD_CODE, CONST_CS | CONST_PERSISTENT);
 
 	XG(breakpoint_count) = 0;
+	XG(output_is_tty) = OUTPUT_NOT_CHECKED;
+
 #ifndef ZTS
 	if (sapi_module.header_handler != xdebug_header_handler) {
 		xdebug_orig_header_handler = sapi_module.header_handler;
 		sapi_module.header_handler = xdebug_header_handler;
 	}
+
+	if (sapi_module.ub_write != xdebug_ub_write) {
+		xdebug_orig_ub_write = sapi_module.ub_write;
+		sapi_module.ub_write = xdebug_ub_write;
+	}
 #endif
 
 	return SUCCESS;
@@ -697,20 +782,40 @@ static void xdebug_stack_element_dtor(void *dummy, void *elem)
 #define COOKIE_ENCODE
 #endif
 
+int xdebug_ub_write(const char *string, unsigned int length TSRMLS_DC)
+{
+	if (XG(remote_enabled)) {
+		if (-1 == XG(context).handler->remote_stream_output(string, length TSRMLS_CC)) {
+			return 0;
+		}
+	}
+	return xdebug_orig_ub_write(string, length TSRMLS_CC);
+}
+
 PHP_RINIT_FUNCTION(xdebug)
 {
 	zend_function *orig;
 	char *idekey;
 	zval **dummy;
-	
-	/* get xdebug ini entries from the environment also */
+
+	/* Get the ide key for this session */
 	XG(ide_key) = NULL;
-	xdebug_env_config();
-	idekey = zend_ini_string("xdebug.idekey", sizeof("xdebug.idekey"), 0);
+	idekey = xdebug_env_key(TSRMLS_C);
+	if (idekey && *idekey) {
+		if (XG(ide_key)) {
+			xdfree(XG(ide_key));
+		}
+		XG(ide_key) = xdstrdup(idekey);
+	}
+
+	/* Get xdebug ini entries from the environment also,
+	   this can override the idekey if one is set */
+	xdebug_env_config(TSRMLS_C);
 
 	XG(no_exec)       = 0;
 	XG(level)         = 0;
 	XG(do_trace)      = 0;
+	XG(coverage_enable) = 0;
 	XG(do_code_coverage) = 0;
 	XG(code_coverage) = xdebug_hash_alloc(32, xdebug_coverage_file_dtor);
 	XG(stack)         = xdebug_llist_alloc(xdebug_stack_element_dtor);
@@ -728,13 +833,6 @@ PHP_RINIT_FUNCTION(xdebug)
 	XG(collected_errors)  = xdebug_llist_alloc(xdebug_llist_string_dtor);
 	XG(reserved_offset) = zend_xdebug_global_offset;
 
-	if (idekey && *idekey) {
-		if (XG(ide_key)) {
-			xdfree(XG(ide_key));
-		}
-		XG(ide_key) = xdstrdup(idekey);
-	}
-
 /* {{{ Initialize auto globals in Zend Engine 2 */
 	zend_is_auto_global("_ENV",     sizeof("_ENV")-1     TSRMLS_CC);
 	zend_is_auto_global("_GET",     sizeof("_GET")-1     TSRMLS_CC);
@@ -780,7 +878,10 @@ PHP_RINIT_FUNCTION(xdebug)
 	XG(remote_enabled) = 0;
 	XG(profiler_enabled) = 0;
 	XG(breakpoints_allowed) = 1;
-	if (XG(auto_trace) && XG(trace_output_dir) && strlen(XG(trace_output_dir))) {
+	if (
+		(XG(auto_trace) || xdebug_trigger_enabled(XG(trace_enable_trigger), "XDEBUG_TRACE" TSRMLS_CC))
+		&& XG(trace_output_dir) && strlen(XG(trace_output_dir))
+	) {
 		/* In case we do an auto-trace we are not interested in the return
 		 * value, but we still have to free it. */
 		xdfree(xdebug_start_trace(NULL, XG(trace_options) TSRMLS_CC));
@@ -817,6 +918,9 @@ PHP_RINIT_FUNCTION(xdebug)
 
 	XG(headers) = xdebug_llist_alloc(xdebug_llist_string_dtor);
 
+	/* Signal that we're in a request now */
+	XG(in_execution) = 1;
+
 	return SUCCESS;
 }
 
@@ -855,6 +959,7 @@ ZEND_MODULE_POST_ZEND_DEACTIVATE_D(xdebug)
 
 	XG(level)            = 0;
 	XG(do_trace)         = 0;
+	XG(coverage_enable)  = 0;
 	XG(do_code_coverage) = 0;
 
 	xdebug_hash_destroy(XG(code_coverage));
@@ -890,6 +995,14 @@ ZEND_MODULE_POST_ZEND_DEACTIVATE_D(xdebug)
 	return SUCCESS;
 }
 
+PHP_RSHUTDOWN_FUNCTION(xdebug)
+{
+	/* Signal that we're no longer in a request */
+	XG(in_execution) = 0;
+
+	return SUCCESS;
+}
+
 
 PHP_MINFO_FUNCTION(xdebug)
 {
@@ -898,6 +1011,7 @@ PHP_MINFO_FUNCTION(xdebug)
 	php_info_print_table_start();
 	php_info_print_table_header(2, "xdebug support", "enabled");
 	php_info_print_table_row(2, "Version", XDEBUG_VERSION);
+	php_info_print_table_row(2, "IDE Key", XG(ide_key));
 	php_info_print_table_end();
 	
 	if (zend_xdebug_initialised == 0) {
@@ -917,6 +1031,32 @@ PHP_MINFO_FUNCTION(xdebug)
 	DISPLAY_INI_ENTRIES();
 }
 
+static int xdebug_trigger_enabled(int setting, char *var_name TSRMLS_DC)
+{
+	zval **dummy;
+
+	if (!setting) {
+		return 0;
+	}
+
+	if (
+		(
+			PG(http_globals)[TRACK_VARS_GET] &&
+			zend_hash_find(PG(http_globals)[TRACK_VARS_GET]->value.ht, var_name, strlen(var_name) + 1, (void **) &dummy) == SUCCESS
+		) || (
+			PG(http_globals)[TRACK_VARS_POST] &&
+			zend_hash_find(PG(http_globals)[TRACK_VARS_POST]->value.ht, var_name, strlen(var_name) + 1, (void **) &dummy) == SUCCESS
+		) || (
+			PG(http_globals)[TRACK_VARS_COOKIE] &&
+			zend_hash_find(PG(http_globals)[TRACK_VARS_COOKIE]->value.ht, var_name, strlen(var_name) + 1, (void **) &dummy) == SUCCESS
+		)
+	) {
+		return 1;
+	}
+
+	return 0;
+}
+
 static void add_used_variables(function_stack_entry *fse, zend_op_array *op_array)
 {
 	int i = 0; 
@@ -939,16 +1079,16 @@ static void add_used_variables(function_stack_entry *fse, zend_op_array *op_arra
 	}
 
 	/* opcode scanning time */
-	while (i < op_array->size) {
+	while (i < op_array->last) {
 		char *cv = NULL;
 		int cv_len;
 
-		if (op_array->opcodes[i].op1.op_type == IS_CV) {
-			cv = zend_get_compiled_variable_name(op_array, op_array->opcodes[i].op1.u.var, &cv_len);
+		if (op_array->opcodes[i].XDEBUG_TYPE(op1) == IS_CV) {
+			cv = (char *) zend_get_compiled_variable_name(op_array, op_array->opcodes[i].XDEBUG_ZNODE_ELEM(op1,var), &cv_len);
 			xdebug_llist_insert_next(fse->used_vars, XDEBUG_LLIST_TAIL(fse->used_vars), xdstrdup(cv));
 		}
-		if (op_array->opcodes[i].op2.op_type == IS_CV) {
-			cv = zend_get_compiled_variable_name(op_array, op_array->opcodes[i].op2.u.var, &cv_len);
+		if (op_array->opcodes[i].XDEBUG_TYPE(op2) == IS_CV) {
+			cv = (char *) zend_get_compiled_variable_name(op_array, op_array->opcodes[i].XDEBUG_ZNODE_ELEM(op2,var), &cv_len);
 			xdebug_llist_insert_next(fse->used_vars, XDEBUG_LLIST_TAIL(fse->used_vars), xdstrdup(cv));
 		}
 		i++;
@@ -1005,7 +1145,13 @@ static void xdebug_throw_exception_hook(zval *exception TSRMLS_DC)
 			xdebug_log_stack(exception_ce->name, Z_STRVAL_P(message), Z_STRVAL_P(file), Z_LVAL_P(line) TSRMLS_CC);
 		}
 		if (PG(display_errors)) {
-			php_printf("%s", exception_trace);
+			xdebug_str tmp_str = { 0, 0, NULL };
+			xdebug_append_error_head(&tmp_str, PG(html_errors), "exception" TSRMLS_CC);
+			xdebug_str_add(&tmp_str, exception_trace, 0);
+			xdebug_append_error_footer(&tmp_str, PG(html_errors) TSRMLS_CC);
+
+			php_printf("%s", tmp_str.d);
+			xdebug_str_dtor(tmp_str);
 		}
 	}
 
@@ -1014,9 +1160,9 @@ static void xdebug_throw_exception_hook(zval *exception TSRMLS_DC)
 
 	if (XG(remote_enabled)) {
 		/* Check if we have a breakpoint on this exception */
-		if (xdebug_hash_find(XG(context).exception_breakpoints, exception_ce->name, strlen(exception_ce->name), (void *) &extra_brk_info)) {
+		if (xdebug_hash_find(XG(context).exception_breakpoints, (char *) exception_ce->name, strlen(exception_ce->name), (void *) &extra_brk_info)) {
 			if (xdebug_handle_hit_value(extra_brk_info)) {
-				if (!XG(context).handler->remote_breakpoint(&(XG(context)), XG(stack), Z_STRVAL_P(file), Z_LVAL_P(line), XDEBUG_BREAK, exception_ce->name, Z_STRVAL_P(message))) {
+				if (!XG(context).handler->remote_breakpoint(&(XG(context)), XG(stack), Z_STRVAL_P(file), Z_LVAL_P(line), XDEBUG_BREAK, (char *) exception_ce->name, Z_STRVAL_P(message))) {
 					XG(remote_enabled) = 0;
 				}
 			}
@@ -1075,7 +1221,8 @@ void xdebug_execute(zend_op_array *op_array TSRMLS_DC)
 	int                   do_return = (XG(do_trace) && XG(trace_file));
 	int                   function_nr = 0;
 	xdebug_llist_element *le;
-	int                   eval_id = 0;
+	int                   eval_id = 0, clear = 0;
+	zval                 *return_val = NULL;
 
 	/* If we're evaluating for the debugger's eval capability, just bail out */
 	if (op_array && op_array->filename && strcmp("xdebug://debug-eval", op_array->filename) == 0) {
@@ -1099,7 +1246,7 @@ void xdebug_execute(zend_op_array *op_array TSRMLS_DC)
 		XG(context).program_name = xdstrdup(op_array->filename);
 	}
 
-	if (XG(level) == 0) {
+	if (XG(level) == 0 && XG(in_execution)) {
 		/* Set session cookie if requested */
 		if (
 			((
@@ -1163,34 +1310,7 @@ void xdebug_execute(zend_op_array *op_array TSRMLS_DC)
 			XG(remote_enable) &&
 			(XG(remote_mode) == XDEBUG_REQ)
 		) {
-			/* Initialize debugging session */
-			if (XG(remote_connect_back)) {
-				zval **remote_addr = NULL;
-				zend_hash_find(Z_ARRVAL_P(PG(http_globals)[TRACK_VARS_SERVER]), "REMOTE_ADDR", 12, (void**)&remote_addr);
-				if (remote_addr) {
-					XG(context).socket = xdebug_create_socket(Z_STRVAL_PP(remote_addr), XG(remote_port));
-				} else {
-					XG(context).socket = xdebug_create_socket(XG(remote_host), XG(remote_port));
-				}
-			} else {
-				XG(context).socket = xdebug_create_socket(XG(remote_host), XG(remote_port));
-			}
-			if (XG(context).socket >= 0) {
-				XG(remote_enabled) = 1;
-
-				/* Get handler from mode */
-				XG(context).handler = xdebug_handler_get(XG(remote_handler));
-				if (!XG(context).handler) {
-					XG(remote_enabled) = 0;
-					zend_error(E_WARNING, "The remote debug handler '%s' is not supported.", XG(remote_handler));
-				} else if (!XG(context).handler->remote_init(&(XG(context)), XDEBUG_REQ)) {
-					/* The request could not be started, ignore it then */
-					XG(remote_enabled) = 0;
-				} else {
-					/* All is well, turn off script time outs */
-					zend_alter_ini_entry("max_execution_time", sizeof("max_execution_time"), "0", strlen("0"), PHP_INI_SYSTEM, PHP_INI_STAGE_ACTIVATE);
-				}
-			}
+			xdebug_init_debugger(TSRMLS_C);
 		}
 		if (magic_cookie) {
 			xdfree(magic_cookie);
@@ -1200,27 +1320,9 @@ void xdebug_execute(zend_op_array *op_array TSRMLS_DC)
 		/* Check for special GET/POST parameter to start profiling */
 		if (
 			!XG(profiler_enabled) &&
-			(
-				XG(profiler_enable)
-				|| 
-				(
-					XG(profiler_enable_trigger) &&
-					(
-						(
-							PG(http_globals)[TRACK_VARS_GET] && 
-							zend_hash_find(PG(http_globals)[TRACK_VARS_GET]->value.ht, "XDEBUG_PROFILE", sizeof("XDEBUG_PROFILE"), (void **) &dummy) == SUCCESS
-						) || (
-							PG(http_globals)[TRACK_VARS_POST] && 
-							zend_hash_find(PG(http_globals)[TRACK_VARS_POST]->value.ht, "XDEBUG_PROFILE", sizeof("XDEBUG_PROFILE"), (void **) &dummy) == SUCCESS
-						) || (
-							PG(http_globals)[TRACK_VARS_COOKIE] && 
-							zend_hash_find(PG(http_globals)[TRACK_VARS_COOKIE]->value.ht, "XDEBUG_PROFILE", sizeof("XDEBUG_PROFILE"), (void **) &dummy) == SUCCESS
-						)
-					)
-				)
-			)
+			(XG(profiler_enable) || xdebug_trigger_enabled(XG(profiler_enable_trigger), "XDEBUG_PROFILE" TSRMLS_CC))
 		) {
-			if (xdebug_profiler_init(op_array->filename TSRMLS_CC) == SUCCESS) {
+			if (xdebug_profiler_init((char *) op_array->filename TSRMLS_CC) == SUCCESS) {
 				XG(profiler_enabled) = 1;
 			}
 		}
@@ -1232,6 +1334,10 @@ void xdebug_execute(zend_op_array *op_array TSRMLS_DC)
 	}
 
 	fse = xdebug_add_stack_frame(edata, op_array, XDEBUG_EXTERNAL TSRMLS_CC);
+	/* A hack to make __call work with profiles. The function *is* user defined after all. */
+	if (fse && fse->prev && fse->function.function && (strcmp(fse->function.function, "__call") == 0)) {
+		fse->prev->user_defined = XDEBUG_EXTERNAL;
+	}
 
 	function_nr = XG(function_count);
 	xdebug_trace_function_begin(fse, function_nr TSRMLS_CC);
@@ -1277,6 +1383,11 @@ void xdebug_execute(zend_op_array *op_array TSRMLS_DC)
 	if (XG(profiler_enabled)) {
 		xdebug_profiler_function_user_begin(fse TSRMLS_CC);
 	}
+
+	if (!EG(return_value_ptr_ptr)) {
+		EG(return_value_ptr_ptr) = &return_val;
+		clear = 1;
+	}
 	xdebug_old_execute(op_array TSRMLS_CC);
 
 	if (XG(profiler_enabled)) {
@@ -1294,6 +1405,10 @@ void xdebug_execute(zend_op_array *op_array TSRMLS_DC)
 			xdfree(t);
 		}
 	}
+	if (clear && *EG(return_value_ptr_ptr)) {
+		zval_ptr_dtor(EG(return_value_ptr_ptr));
+		EG(return_value_ptr_ptr) = NULL;
+	}
 
 	/* Check for return breakpoints */
 	if (XG(remote_enabled) && XG(breakpoints_allowed)) {
@@ -1315,7 +1430,7 @@ static int check_soap_call(function_stack_entry *fse)
 	if (fse->function.class && 
 		(
 			(strstr(fse->function.class, "SoapClient") != NULL) ||
-			(strstr(fse->function.class, "SoapClient") != NULL)
+			(strstr(fse->function.class, "SoapServer") != NULL)
 		) &&
 		(zend_hash_find(&module_registry, "soap", 5, (void**) &tmp_mod_entry) == SUCCESS)
 	) {
@@ -1333,7 +1448,7 @@ void xdebug_execute_internal(zend_execute_data *current_execute_data, int return
 	int                   function_nr = 0;
 
 	int                   restore_error_handler_situation = 0;
-	void                (*tmp_error_cb)(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args);
+	void                (*tmp_error_cb)(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args) = NULL;
 
 	XG(level)++;
 	if (XG(level) == XG(max_nesting_level)) {
@@ -1383,11 +1498,13 @@ void xdebug_execute_internal(zend_execute_data *current_execute_data, int return
 	if (XG(collect_return) && do_return && XG(do_trace) && XG(trace_file)) {
 		cur_opcode = *EG(opline_ptr);
 		if (cur_opcode) {
-			zval *ret = xdebug_zval_ptr(&(cur_opcode->result), current_execute_data->Ts TSRMLS_CC);
-			char* t = xdebug_return_trace_stack_retval(fse, ret TSRMLS_CC);
-			fprintf(XG(trace_file), "%s", t);
-			fflush(XG(trace_file));
-			xdfree(t);
+			zval *ret = xdebug_zval_ptr(cur_opcode->XDEBUG_TYPE(result), &(cur_opcode->result), current_execute_data->Ts TSRMLS_CC);
+			if (ret) {
+				char* t = xdebug_return_trace_stack_retval(fse, ret TSRMLS_CC);
+				fprintf(XG(trace_file), "%s", t);
+				fflush(XG(trace_file));
+				xdfree(t);
+			}
 		}
 	}
 
@@ -1412,7 +1529,6 @@ int xdebug_exit_handler(ZEND_OPCODE_HANDLER_ARGS)
 	return ZEND_USER_OPCODE_DISPATCH;
 }
 
-
 /* {{{ zend_op_array srm_compile_file (file_handle, type)
  *    This function provides a hook for the execution of bananas */
 zend_op_array *xdebug_compile_file(zend_file_handle *file_handle, int type TSRMLS_DC)
@@ -1422,7 +1538,7 @@ zend_op_array *xdebug_compile_file(zend_file_handle *file_handle, int type TSRML
 	op_array = old_compile_file(file_handle, type TSRMLS_CC);
 
 	if (op_array) {
-		if (XG(do_code_coverage) && XG(code_coverage_unused && op_array->done_pass_two)) {
+		if (XG(do_code_coverage) && XG(code_coverage_unused && XDEBUG_PASS_TWO_DONE)) {
 			xdebug_prefill_code_coverage(op_array TSRMLS_CC);
 		}
 	}
@@ -1433,7 +1549,21 @@ zend_op_array *xdebug_compile_file(zend_file_handle *file_handle, int type TSRML
 static int xdebug_header_handler(sapi_header_struct *h XG_SAPI_HEADER_OP_DC, sapi_headers_struct *s TSRMLS_DC)
 {
 	if (XG(headers)) {
+#if PHP_VERSION_ID >= 50300
+		switch (op) {
+			case SAPI_HEADER_ADD:
+			case SAPI_HEADER_REPLACE:
+				xdebug_llist_insert_next(XG(headers), XDEBUG_LLIST_TAIL(XG(headers)), xdstrdup(h->header));
+			break;
+			case SAPI_HEADER_DELETE_ALL:
+				xdebug_llist_empty(XG(headers), NULL);
+			case SAPI_HEADER_DELETE:
+			case SAPI_HEADER_SET_STATUS:
+			break;
+		}
+#else
 		xdebug_llist_insert_next(XG(headers), XDEBUG_LLIST_TAIL(XG(headers)), xdstrdup(h->header));
+#endif
 	}
 	if (xdebug_orig_header_handler) {
 		return xdebug_orig_header_handler(h XG_SAPI_HEADER_OP_CC, s TSRMLS_CC);
@@ -1470,12 +1600,23 @@ PHP_FUNCTION(xdebug_var_dump)
 	}
 	
 	for (i = 0; i < argc; i++) {
-		if (PG(html_errors)) {
+		if (XG(default_enable) == 0) {
+			xdebug_php_var_dump(args[i], 1 TSRMLS_CC);
+		}
+		else if (PG(html_errors)) {
 			val = xdebug_get_zval_value_fancy(NULL, (zval*) *args[i], &len, 0, NULL TSRMLS_CC);
 			PHPWRITE(val, len);
 			xdfree(val);
-		} else {
-			xdebug_php_var_dump(args[i], 1 TSRMLS_CC);
+		}
+		else if ((XG(cli_color) == 1 && xdebug_is_output_tty(TSRMLS_C)) || (XG(cli_color) == 2)) {
+			val = xdebug_get_zval_value_ansi((zval*) *args[i], 0, NULL);
+			PHPWRITE(val, strlen(val));
+			xdfree(val);
+		} 
+		else {
+			val = xdebug_get_zval_value_text((zval*) *args[i], 0, NULL);
+			PHPWRITE(val, strlen(val));
+			xdfree(val);
 		}
 	}
 	
@@ -1516,7 +1657,12 @@ PHP_FUNCTION(xdebug_debug_zval)
 				if (PG(html_errors)) {
 					val = xdebug_get_zval_value_fancy(NULL, debugzval, &len, 1, NULL TSRMLS_CC);
 					PHPWRITE(val, len);
-				} else {
+				}
+				else if ((XG(cli_color) == 1 && xdebug_is_output_tty(TSRMLS_C)) || (XG(cli_color) == 2)) {
+					val = xdebug_get_zval_value_ansi(debugzval, 1, NULL);
+					PHPWRITE(val, strlen(val));
+				}
+				else {
 					val = xdebug_get_zval_value(debugzval, 1, NULL);
 					PHPWRITE(val, strlen(val));
 				}
@@ -1711,7 +1857,6 @@ ZEND_DLEXPORT void xdebug_statement_call(zend_op_array *op_array)
 	function_stack_entry *fse;
 	int                   lineno;
 	char                 *file;
-	int                   file_len = 0;
 	int                   level = 0;
 	TSRMLS_FETCH();
 
@@ -1721,8 +1866,7 @@ ZEND_DLEXPORT void xdebug_statement_call(zend_op_array *op_array)
 
 	lineno = EG(current_execute_data)->opline->lineno;
 
-	file = op_array->filename;
-	file_len = strlen(file);
+	file = (char *) op_array->filename;
 
 	if (XG(do_code_coverage)) {
 		xdebug_count_line(file, lineno, 0, 0 TSRMLS_CC);
@@ -1782,6 +1926,7 @@ ZEND_DLEXPORT void xdebug_statement_call(zend_op_array *op_array)
 			int   break_ok;
 			int   old_error_reporting;
 			zval  retval;
+			int   file_len = strlen(file);
 
 			for (le = XDEBUG_LLIST_HEAD(XG(context).line_breakpoints); le != NULL; le = XDEBUG_LLIST_NEXT(le)) {
 				brk = XDEBUG_LLIST_VALP(le);
