@@ -61,7 +61,7 @@ int xdebug_common_override_handler(ZEND_OPCODE_HANDLER_ARGS)
 		cur_opcode = *EG(opline_ptr);
 		lineno = cur_opcode->lineno;
 
-		file = op_array->filename;
+		file = (char *)op_array->filename;
 
 		xdebug_count_line(file, lineno, 0, 0 TSRMLS_CC);
 	}
@@ -83,44 +83,56 @@ static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
 	next_opcode = cur_opcode + 1;
 	prev_opcode = cur_opcode - 1;
 
-	if (cur_opcode->op1.op_type == IS_VAR &&
-			(next_opcode->op1.op_type == IS_VAR || cur_opcode->op2.op_type == IS_VAR) &&
+	if (cur_opcode->XDEBUG_TYPE(op1) == IS_VAR &&
+			(next_opcode->XDEBUG_TYPE(op1) == IS_VAR || cur_opcode->XDEBUG_TYPE(op2) == IS_VAR) &&
 			prev_opcode->opcode == ZEND_FETCH_RW &&
-			prev_opcode->op1.op_type == IS_CONST &&
+			prev_opcode->XDEBUG_TYPE(op1) == IS_CONST &&
+#if PHP_VERSION_ID >= 50399
+			Z_TYPE_P(prev_opcode->op1.zv) == IS_STRING
+#else
 			prev_opcode->op1.u.constant.type == IS_STRING
+#endif
 	) {
+#if PHP_VERSION_ID >= 50399
+		xdebug_str_add(&name, xdebug_sprintf("$%s", Z_STRVAL_P(prev_opcode->op1.zv)), 1);
+#else
 		xdebug_str_add(&name, xdebug_sprintf("$%s", prev_opcode->op1.u.constant.value.str.val), 1);
+#endif
 	}
 
-	is_static = (prev_opcode->op1.op_type == IS_CONST && prev_opcode->op2.u.EA.type == ZEND_FETCH_STATIC_MEMBER);
+	is_static = (prev_opcode->XDEBUG_TYPE(op1) == IS_CONST && prev_opcode->XDEBUG_EXTENDED_VALUE(op2) == ZEND_FETCH_STATIC_MEMBER);
 	options = xdebug_var_export_options_from_ini(TSRMLS_C);
 	options->no_decoration = 1;
 
-	if (cur_opcode->op1.op_type == IS_CV) {
-		xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, cur_opcode->op1.u.var, &cv_len)), 1);
-	} else if (cur_opcode->op1.op_type == IS_VAR && cur_opcode->opcode == ZEND_ASSIGN && prev_opcode->opcode == ZEND_FETCH_W) {
+	if (cur_opcode->XDEBUG_TYPE(op1) == IS_CV) {
+		xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, cur_opcode->XDEBUG_ZNODE_ELEM(op1, var), &cv_len)), 1);
+	} else if (cur_opcode->XDEBUG_TYPE(op1) == IS_VAR && cur_opcode->opcode == ZEND_ASSIGN && prev_opcode->opcode == ZEND_FETCH_W) {
 		if (is_static) {
 			xdebug_str_add(&name, xdebug_sprintf("self::"), 1);
 		} else {
-			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &prev_opcode->op1, execute_data->Ts, &is_var), 0, options);
+			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, prev_opcode->XDEBUG_TYPE(op1), &prev_opcode->op1, execute_data->Ts, &is_var), 0, options);
 			xdebug_str_add(&name, xdebug_sprintf("$%s", zval_value), 1);
 		}
-	} else if (is_static) { // todo : see if you can change this and the previous cases around
+	} else if (is_static) { /* todo : see if you can change this and the previous cases around */
 		xdebug_str_add(&name, xdebug_sprintf("self::"), 1 );
 	}
 	if (cur_opcode->opcode >= ZEND_ASSIGN_ADD && cur_opcode->opcode <= ZEND_ASSIGN_BW_XOR ) {
 		if (cur_opcode->extended_value == ZEND_ASSIGN_OBJ) {
-			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &cur_opcode->op2, execute_data->Ts, &is_var), 0, options);
-			if (cur_opcode->op1.op_type == IS_UNUSED) {
+			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op2), &cur_opcode->op2, execute_data->Ts, &is_var), 0, options);
+			if (cur_opcode->XDEBUG_TYPE(op1) == IS_UNUSED) {
 				xdebug_str_add(&name, xdebug_sprintf("$this->%s", zval_value), 1);
 			} else {
 				xdebug_str_add(&name, xdebug_sprintf("->%s", zval_value), 1);
 			}
 		} else if (cur_opcode->extended_value == ZEND_ASSIGN_DIM) {
-			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &cur_opcode->op2, execute_data->Ts, &is_var), 0, NULL);
+			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op2), &cur_opcode->op2, execute_data->Ts, &is_var), 0, NULL);
 			xdebug_str_add(&name,xdebug_sprintf("[%s]", zval_value), 1);
 		}
 	}
+	if (cur_opcode->opcode >= ZEND_PRE_INC_OBJ && cur_opcode->opcode <= ZEND_POST_DEC_OBJ) {
+		zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op2), &cur_opcode->op2, execute_data->Ts, &is_var), 0, options);
+		xdebug_str_add(&name, xdebug_sprintf("$this->%s", zval_value), 1);
+	}
 	if (zval_value) {
 		xdfree(zval_value);
 		zval_value = NULL;
@@ -129,7 +141,7 @@ static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
 	/* Scroll back to start of FETCHES */
 	gohungfound = 0;
 	opcode_ptr = prev_opcode;
-	while (opcode_ptr->opcode == ZEND_FETCH_DIM_W || opcode_ptr->opcode == ZEND_FETCH_OBJ_W || opcode_ptr->opcode == ZEND_FETCH_W) {
+	while (opcode_ptr->opcode == ZEND_FETCH_DIM_W || opcode_ptr->opcode == ZEND_FETCH_OBJ_W || opcode_ptr->opcode == ZEND_FETCH_W || opcode_ptr->opcode == ZEND_FETCH_RW) {
 		opcode_ptr = opcode_ptr - 1;
 		gohungfound = 1;
 	}
@@ -138,21 +150,29 @@ static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
 	if (gohungfound) {
 		do
 		{
-			if (opcode_ptr->op1.op_type == IS_UNUSED && opcode_ptr->opcode == ZEND_FETCH_OBJ_W) {
+			if (opcode_ptr->XDEBUG_TYPE(op1) == IS_UNUSED && opcode_ptr->opcode == ZEND_FETCH_OBJ_W) {
 				xdebug_str_add(&name, "$this", 0);
 			}
-			if (opcode_ptr->op1.op_type == IS_CV) {
-				xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, opcode_ptr->op1.u.var, &cv_len)), 1);
+			if (opcode_ptr->XDEBUG_TYPE(op1) == IS_CV) {
+				xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, opcode_ptr->XDEBUG_ZNODE_ELEM(op1, var), &cv_len)), 1);
 			}
 			if (opcode_ptr->opcode == ZEND_FETCH_W) {
-				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &opcode_ptr->op1, execute_data->Ts, &is_var), 0, options);
+				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->XDEBUG_TYPE(op1), &opcode_ptr->op1, execute_data->Ts, &is_var), 0, options);
+				xdebug_str_add(&name, xdebug_sprintf("%s", zval_value), 1);
+			}
+			if (is_static && opcode_ptr->opcode == ZEND_FETCH_RW) {
+				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->XDEBUG_TYPE(op1), &opcode_ptr->op1, execute_data->Ts, &is_var), 0, options);
 				xdebug_str_add(&name, xdebug_sprintf("%s", zval_value), 1);
 			}
 			if (opcode_ptr->opcode == ZEND_FETCH_DIM_W) {
-				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &opcode_ptr->op2, execute_data->Ts, &is_var), 0, NULL);
-				xdebug_str_add(&name, xdebug_sprintf("[%s]", zval_value), 1);
+				if (opcode_ptr->XDEBUG_TYPE(op2) != IS_VAR) {
+					zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->XDEBUG_TYPE(op2), &opcode_ptr->op2, execute_data->Ts, &is_var), 0, NULL);
+					xdebug_str_add(&name, xdebug_sprintf("[%s]", zval_value), 1);
+				} else {
+					xdebug_str_add(&name, xdebug_sprintf("[???]") , 1);
+				}
 			} else if (opcode_ptr->opcode == ZEND_FETCH_OBJ_W) {
-				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &opcode_ptr->op2, execute_data->Ts, &is_var), 0, options);
+				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->XDEBUG_TYPE(op2), &opcode_ptr->op2, execute_data->Ts, &is_var), 0, options);
 				xdebug_str_add(&name, xdebug_sprintf("->%s", zval_value), 1);
 			}
 			opcode_ptr = opcode_ptr + 1;
@@ -160,22 +180,22 @@ static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
 				xdfree(zval_value);
 				zval_value = NULL;
 			}
-		} while (opcode_ptr->opcode == ZEND_FETCH_DIM_W || opcode_ptr->opcode == ZEND_FETCH_OBJ_W || opcode_ptr->opcode == ZEND_FETCH_W);
+		} while (opcode_ptr->opcode == ZEND_FETCH_DIM_W || opcode_ptr->opcode == ZEND_FETCH_OBJ_W || opcode_ptr->opcode == ZEND_FETCH_W || opcode_ptr->opcode == ZEND_FETCH_RW);
 	}
 
 	if (cur_opcode->opcode == ZEND_ASSIGN_OBJ) {
-		if (cur_opcode->op1.op_type == IS_UNUSED) {
+		if (cur_opcode->XDEBUG_TYPE(op1) == IS_UNUSED) {
 			xdebug_str_add(&name, "$this", 0);
 		}
-		dimval = xdebug_get_zval(execute_data, &cur_opcode->op2, execute_data->Ts, &is_var);
+		dimval = xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op2), &cur_opcode->op2, execute_data->Ts, &is_var);
 		xdebug_str_add(&name, xdebug_sprintf("->%s", Z_STRVAL_P(dimval)), 1);
 	}
 
 	if (cur_opcode->opcode == ZEND_ASSIGN_DIM) {
-		if (next_opcode->opcode == ZEND_OP_DATA && cur_opcode->op2.op_type == IS_UNUSED) {
+		if (next_opcode->opcode == ZEND_OP_DATA && cur_opcode->XDEBUG_TYPE(op2) == IS_UNUSED) {
 			xdebug_str_add(&name, "[]", 0);
 		} else {
-			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &opcode_ptr->op2, execute_data->Ts, &is_var), 0, NULL);
+			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->XDEBUG_TYPE(op2), &opcode_ptr->op2, execute_data->Ts, &is_var), 0, NULL);
 			xdebug_str_add(&name, xdebug_sprintf("[%s]", zval_value), 1);
 			xdfree(zval_value);
 		}
@@ -201,7 +221,7 @@ static int xdebug_common_assign_dim_handler(char *op, int do_cc, ZEND_OPCODE_HAN
 
 	cur_opcode = *EG(opline_ptr);
 	next_opcode = cur_opcode + 1;
-	file = op_array->filename;
+	file = (char *) op_array->filename;
 	lineno = cur_opcode->lineno;
 
 	if (do_cc && XG(do_code_coverage)) {
@@ -222,11 +242,24 @@ static int xdebug_common_assign_dim_handler(char *op, int do_cc, ZEND_OPCODE_HAN
 			xdfree(full_varname);
 			full_varname = tmp_varname;
 
-			val = xdebug_get_zval(execute_data, &cur_opcode->op1, execute_data->Ts, &is_var);
+			val = xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op1), &cur_opcode->op1, execute_data->Ts, &is_var);
+		} else if (cur_opcode->opcode >= ZEND_PRE_INC_OBJ && cur_opcode->opcode <= ZEND_POST_DEC_OBJ) {
+			char *tmp_varname;
+
+			switch (cur_opcode->opcode) {
+				case ZEND_PRE_INC_OBJ:  tmp_varname = xdebug_sprintf("++%s", full_varname); break;
+				case ZEND_POST_INC_OBJ: tmp_varname = xdebug_sprintf("%s++", full_varname); break;
+				case ZEND_PRE_DEC_OBJ:  tmp_varname = xdebug_sprintf("--%s", full_varname); break;
+				case ZEND_POST_DEC_OBJ: tmp_varname = xdebug_sprintf("%s--", full_varname); break;
+			}
+			xdfree(full_varname);
+			full_varname = tmp_varname;
+
+			val = xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op2), &cur_opcode->op2, execute_data->Ts, &is_var);
 		} else if (next_opcode->opcode == ZEND_OP_DATA) {
-			val = xdebug_get_zval(execute_data, &next_opcode->op1, execute_data->Ts, &is_var);
+			val = xdebug_get_zval(execute_data, next_opcode->XDEBUG_TYPE(op1), &next_opcode->op1, execute_data->Ts, &is_var);
 		} else {
-			val = xdebug_get_zval(execute_data, &cur_opcode->op2, execute_data->Ts, &is_var);
+			val = xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op2), &cur_opcode->op2, execute_data->Ts, &is_var);
 		}
 
 		fse = XDEBUG_LLIST_VALP(XDEBUG_LLIST_TAIL(XG(stack)));
@@ -257,6 +290,10 @@ XDEBUG_OPCODE_OVERRIDE_ASSIGN(pre_inc,"",0)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(post_inc,"",0)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(pre_dec,"",0)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(post_dec,"",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(pre_inc_obj,"",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(post_inc_obj,"",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(pre_dec_obj,"",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(post_dec_obj,"",0)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_concat,".=",1)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_bw_or,"|=",0)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_bw_and,"&=",0)
@@ -268,29 +305,32 @@ void xdebug_count_line(char *filename, int lineno, int executable, int deadcode
 {
 	xdebug_coverage_file *file;
 	xdebug_coverage_line *line;
-	char *sline;
 
-	sline = xdebug_sprintf("%d", lineno);
-
-	/* Check if the file already exists in the hash */
-	if (!xdebug_hash_find(XG(code_coverage), filename, strlen(filename), (void *) &file)) {
-		/* The file does not exist, so we add it to the hash, and
-		 *  add a line element to the file */
-		file = xdmalloc(sizeof(xdebug_coverage_file));
-		file->name = xdstrdup(filename);
-		file->lines = xdebug_hash_alloc(128, xdebug_coverage_line_dtor);
+	if (strcmp(XG(previous_filename), filename) == 0) {
+		file = XG(previous_file);
+	} else {
+		/* Check if the file already exists in the hash */
+		if (!xdebug_hash_find(XG(code_coverage), filename, strlen(filename), (void *) &file)) {
+			/* The file does not exist, so we add it to the hash, and
+			 *  add a line element to the file */
+			file = xdmalloc(sizeof(xdebug_coverage_file));
+			file->name = xdstrdup(filename);
+			file->lines = xdebug_hash_alloc(128, xdebug_coverage_line_dtor);
 		
-		xdebug_hash_add(XG(code_coverage), filename, strlen(filename), file);
+			xdebug_hash_add(XG(code_coverage), filename, strlen(filename), file);
+		}
+		XG(previous_filename) = file->name;
+		XG(previous_file) = file;
 	}
 
 	/* Check if the line already exists in the hash */
-	if (!xdebug_hash_find(file->lines, sline, strlen(sline), (void *) &line)) {
+	if (!xdebug_hash_index_find(file->lines, lineno, (void *) &line)) {
 		line = xdmalloc(sizeof(xdebug_coverage_line));
 		line->lineno = lineno;
 		line->count = 0;
 		line->executable = 0;
 
-		xdebug_hash_add(file->lines, sline, strlen(sline), line);
+		xdebug_hash_index_add(file->lines, lineno, line);
 	}
 
 	if (executable) {
@@ -302,8 +342,6 @@ void xdebug_count_line(char *filename, int lineno, int executable, int deadcode
 	} else {
 		line->count++;
 	}
-
-	xdfree(sline);
 }
 
 static void prefill_from_opcode(char *fn, zend_op opcode, int deadcode TSRMLS_DC)
@@ -331,7 +369,7 @@ static zend_brk_cont_element* xdebug_find_brk_cont(zval *nest_levels_zval, int a
 
 	do {
 		if (array_offset == -1) {
-			// broken break/continue in code
+			/* broken break/continue in code */
 			return NULL;
 		}
 		jmp_to = &op_array->brk_cont_array[array_offset];
@@ -346,7 +384,7 @@ static int xdebug_find_jump(zend_op_array *opa, unsigned int position, long *jmp
 
 	zend_op opcode = opa->opcodes[position];
 	if (opcode.opcode == ZEND_JMP) {
-		*jmp1 = ((long) opcode.op1.u.jmp_addr - (long) base_address) / sizeof(zend_op);
+		*jmp1 = ((long) opcode.XDEBUG_ZNODE_ELEM(op1, jmp_addr) - (long) base_address) / sizeof(zend_op);
 		return 1;
 	} else if (
 		opcode.opcode == ZEND_JMPZ ||
@@ -355,34 +393,38 @@ static int xdebug_find_jump(zend_op_array *opa, unsigned int position, long *jmp
 		opcode.opcode == ZEND_JMPNZ_EX
 	) {
 		*jmp1 = position + 1;
-		*jmp2 = ((long) opcode.op2.u.jmp_addr - (long) base_address) / sizeof(zend_op);
+		*jmp2 = ((long) opcode.XDEBUG_ZNODE_ELEM(op2, jmp_addr) - (long) base_address) / sizeof(zend_op);
 		return 1;
 	} else if (opcode.opcode == ZEND_JMPZNZ) {
-		*jmp1 = opcode.op2.u.opline_num;
+		*jmp1 = opcode.XDEBUG_ZNODE_ELEM(op2, opline_num);
 		*jmp2 = opcode.extended_value;
 		return 1;
 	} else if (opcode.opcode == ZEND_BRK || opcode.opcode == ZEND_CONT) {
 		zend_brk_cont_element *el;
 
-		if (opcode.op2.op_type == IS_CONST
-		    && opcode.op1.u.jmp_addr != (zend_op*) 0xFFFFFFFF
+		if (opcode.XDEBUG_TYPE(op2) == IS_CONST
+		    && opcode.XDEBUG_ZNODE_ELEM(op1, jmp_addr) != (zend_op*) 0xFFFFFFFF
 		) {
+#if PHP_VERSION_ID >= 50399
+			el = xdebug_find_brk_cont(opcode.op2.zv, opcode.XDEBUG_ZNODE_ELEM(op1, opline_num), opa);
+#else
 			el = xdebug_find_brk_cont(&opcode.op2.u.constant, opcode.op1.u.opline_num, opa);
+#endif
 			if (el) {
 				*jmp1 = opcode.opcode == ZEND_BRK ? el->brk : el->cont;
 				return 1;
 			} else {
-				// broken break/continue in code
+				/* broken break/continue in code */
 				return 0;
 			}
 		}
 	} else if (opcode.opcode == ZEND_FE_RESET || opcode.opcode == ZEND_FE_FETCH) {
 		*jmp1 = position + 1;
-		*jmp2 = opcode.op2.u.opline_num;
+		*jmp2 = opcode.XDEBUG_ZNODE_ELEM(op2, opline_num);
 		return 1;
 #if (PHP_MAJOR_VERSION > 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3)
 	} else if (opcode.opcode == ZEND_GOTO) {
-		*jmp1 = ((long) opcode.op1.u.jmp_addr - (long) base_address) / sizeof(zend_op);
+		*jmp1 = ((long) opcode.XDEBUG_ZNODE_ELEM(op1, jmp_addr) - (long) base_address) / sizeof(zend_op);
 		return 1;
 #endif
 	}
@@ -433,7 +475,12 @@ static void xdebug_analyse_branch(zend_op_array *opa, unsigned int position, xde
 			break;
 		}
 		/* See if we have a return instruction */
-		if (opa->opcodes[position].opcode == ZEND_RETURN) {
+		if (
+			opa->opcodes[position].opcode == ZEND_RETURN
+#if PHP_VERSION_ID >= 50400
+			|| opa->opcodes[position].opcode == ZEND_RETURN_BY_REF
+#endif
+		) {
 			/*(fprintf(stderr, "XDEBUG Return found\n");)*/
 			break;
 		}
@@ -458,33 +505,33 @@ static void xdebug_analyse_oparray(zend_op_array *opa, xdebug_set *set TSRMLS_DC
 	}
 }
 
-static void prefill_from_oparray(char *fn, zend_op_array *opa TSRMLS_DC)
+static void prefill_from_oparray(char *fn, zend_op_array *op_array TSRMLS_DC)
 {
 	unsigned int i;
 	xdebug_set *set = NULL;
 
-	opa->reserved[XG(reserved_offset)] = (void*) 1;
+	op_array->reserved[XG(reserved_offset)] = (void*) 1;
 
 	/* Check for abstract methods and simply return from this function in those
 	 * cases. */
 #if PHP_VERSION_ID >= 50300
-	if (opa->last >= 3 && opa->opcodes[opa->last - 3].opcode == ZEND_RAISE_ABSTRACT_ERROR)
+	if (op_array->last >= 3 && op_array->opcodes[op_array->last - 3].opcode == ZEND_RAISE_ABSTRACT_ERROR)
 #else
-	if (opa->last >= 4 && opa->opcodes[opa->last - 4].opcode == ZEND_RAISE_ABSTRACT_ERROR)
+	if (op_array->last >= 4 && op_array->opcodes[op_array->last - 4].opcode == ZEND_RAISE_ABSTRACT_ERROR)
 #endif
 	{
 		return;
 	}	
 
 	/* Run dead code analysis if requested */
-	if (XG(code_coverage_dead_code_analysis) && opa->done_pass_two) {
-		set = xdebug_set_create(opa->last);
-		xdebug_analyse_oparray(opa, set TSRMLS_CC);
+	if (XG(code_coverage_dead_code_analysis) && XDEBUG_PASS_TWO_DONE) {
+		set = xdebug_set_create(op_array->last);
+		xdebug_analyse_oparray(op_array, set TSRMLS_CC);
 	}
 
 	/* The normal loop then finally */
-	for (i = 0; i < opa->last; i++) {
-		zend_op opcode = opa->opcodes[i];
+	for (i = 0; i < op_array->last; i++) {
+		zend_op opcode = op_array->opcodes[i];
 		prefill_from_opcode(fn, opcode, set ? !xdebug_set_in(set, i) : 0 TSRMLS_CC);
 	}
 
@@ -503,7 +550,7 @@ static int prefill_from_function_table(zend_op_array *opa XDEBUG_ZEND_HASH_APPLY
 	new_filename = va_arg(args, char*);
 	if (opa->type == ZEND_USER_FUNCTION) {
 		if (opa->reserved[XG(reserved_offset)] != (void*) 1 /* && opa->filename && strcmp(opa->filename, new_filename) == 0)*/) {
-			prefill_from_oparray(opa->filename, opa TSRMLS_CC);
+			prefill_from_oparray((char *) opa->filename, opa TSRMLS_CC);
 		}
 	}
 
@@ -531,7 +578,7 @@ static int prefill_from_class_table(zend_class_entry **class_entry XDEBUG_ZEND_H
 void xdebug_prefill_code_coverage(zend_op_array *op_array TSRMLS_DC)
 {
 	if (op_array->reserved[XG(reserved_offset)] != (void*) 1) {
-		prefill_from_oparray(op_array->filename, op_array TSRMLS_CC);
+		prefill_from_oparray((char *) op_array->filename, op_array TSRMLS_CC);
 	}
 
 	zend_hash_apply_with_arguments(CG(function_table)  XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) prefill_from_function_table, 1, op_array->filename);
@@ -548,12 +595,15 @@ PHP_FUNCTION(xdebug_start_code_coverage)
 	XG(code_coverage_unused) = (options & XDEBUG_CC_OPTION_UNUSED);
 	XG(code_coverage_dead_code_analysis) = (options & XDEBUG_CC_OPTION_DEAD_CODE);
 
-	if (XG(extended_info)) {
-		RETVAL_BOOL(!XG(do_code_coverage));
-		XG(do_code_coverage) = 1;
-	} else {
+	if (!XG(extended_info)) {
 		php_error(E_WARNING, "You can only use code coverage when you leave the setting of 'xdebug.extended_info' to the default '1'.");
-		RETVAL_BOOL(0);
+		RETURN_FALSE;
+	} else if (!XG(code_coverage)) {
+		php_error(E_WARNING, "Code coverage needs to be enabled in php.ini by setting 'xdebug.coverage_enable' to '1'.");
+		RETURN_FALSE;
+	} else {
+		XG(do_code_coverage) = 1;
+		RETURN_TRUE;
 	}
 }
 
@@ -566,6 +616,8 @@ PHP_FUNCTION(xdebug_stop_code_coverage)
 	}
 	if (XG(do_code_coverage)) {
 		if (cleanup) {
+			XG(previous_filename) = "";
+			XG(previous_file) = NULL;
 			xdebug_hash_destroy(XG(code_coverage));
 			XG(code_coverage) = xdebug_hash_alloc(32, xdebug_coverage_file_dtor);
 		}
