@@ -61,6 +61,10 @@ XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(pre_inc);
 XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(post_inc);
 XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(pre_dec);
 XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(post_dec);
+XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(pre_inc_obj);
+XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(post_inc_obj);
+XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(pre_dec_obj);
+XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(post_dec_obj);
 XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(assign_concat);
 XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(assign_bw_or);
 XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(assign_bw_and);
