@@ -35,6 +35,7 @@
 #include "xdebug_private.h"
 #include "xdebug_code_coverage.h"
 #include "xdebug_com.h"
+#include "xdebug_compat.h"
 #include "xdebug_handler_dbgp.h"
 #include "xdebug_hash.h"
 #include "xdebug_llist.h"
@@ -273,6 +274,8 @@ static void send_message(xdebug_con *context, xdebug_xml_node *message TSRMLS_DC
 #define XF_ST_ARRAY_INDEX_NUM    1
 #define XF_ST_ARRAY_INDEX_ASSOC  2
 #define XF_ST_OBJ_PROPERTY       3
+#define XF_ST_STATIC_ROOT        4
+#define XF_ST_STATIC_PROPERTY    5
 
 inline static HashTable *fetch_ht_from_zval(zval *z TSRMLS_DC)
 {
@@ -287,27 +290,30 @@ inline static HashTable *fetch_ht_from_zval(zval *z TSRMLS_DC)
 	return NULL;
 }
 
-inline static char *fetch_classname_from_zval(zval *z, int *length TSRMLS_DC)
+inline static char *fetch_classname_from_zval(zval *z, int *length, zend_class_entry **ce TSRMLS_DC)
 {
 	char *name;
 	zend_uint name_len;
+	zend_class_entry *tmp_ce;
 
 	if (Z_TYPE_P(z) != IS_OBJECT) {
 		return NULL;
 	}
 
+	tmp_ce = zend_get_class_entry(z TSRMLS_CC);
 	if (Z_OBJ_HT_P(z)->get_class_name == NULL ||
-		Z_OBJ_HT_P(z)->get_class_name(z, &name, &name_len, 0 TSRMLS_CC) != SUCCESS) {
-		zend_class_entry *ce;
+		Z_OBJ_HT_P(z)->get_class_name(z, (const char **) &name, &name_len, 0 TSRMLS_CC) != SUCCESS) {
 
-		ce = zend_get_class_entry(z TSRMLS_CC);
-		if (!ce) {
+		if (!tmp_ce) {
 			return NULL;
 		}
 
-		*length = ce->name_length;
-		return estrdup(ce->name);
-	} 
+		*length = tmp_ce->name_length;
+		*ce = tmp_ce;
+		return estrdup(tmp_ce->name);
+	} else {
+		*ce = tmp_ce;
+	}
 
 	*length = name_len;
 	return name;
@@ -337,13 +343,49 @@ static char* prepare_search_key(char *name, int *name_length, char *prefix, int
 	return element;
 }
 
-static zval* fetch_zval_from_symbol_table(HashTable *ht, char* name, int name_length, int type, char* ccn, int ccnl TSRMLS_DC)
+static zval* fetch_zval_from_symbol_table(HashTable *ht, char* name, int name_length, int type, char* ccn, int ccnl, zend_class_entry *cce TSRMLS_DC)
 {
 	zval **retval_pp = NULL, *retval_p = NULL;
-	char  *element;
+	char  *element = NULL;
 	int    element_length = name_length;
+#if PHP_VERSION_ID >= 50400
+	zend_property_info *zpp;
+#endif
 
 	switch (type) {
+		case XF_ST_STATIC_ROOT:
+		case XF_ST_STATIC_PROPERTY:
+#if PHP_VERSION_ID < 50400
+			ht = CE_STATIC_MEMBERS(cce);
+			goto continue_from_static_root;
+#else
+			/* First we try a public,private,protected property */
+			element = prepare_search_key(name, &element_length, "", 0);
+			if (cce && &cce->properties_info && zend_hash_find(&cce->properties_info, element, element_length + 1, (void **) &zpp) == SUCCESS) {
+				retval_p = cce->static_members_table[zpp->offset];
+				goto cleanup;
+			}
+			element_length = name_length;
+
+			/* Then we try to see whether the first char is * and use the part between * and * as class name for the private property */
+			if (name[0] == '*') {
+				char *secondStar;
+				
+				secondStar = strstr(name + 1, "*");
+				if (secondStar) {
+					free(element);
+					element_length = name_length - (secondStar + 1 - name);
+					element = prepare_search_key(secondStar + 1, &element_length, "", 0);
+					if (cce && &cce->properties_info && zend_hash_find(&cce->properties_info, element, element_length + 1, (void **) &zpp) == SUCCESS) {
+						retval_p = cce->static_members_table[zpp->offset];
+						goto cleanup;
+					}
+				}
+			}
+
+			break;
+#endif
+
 		case XF_ST_ROOT:
 #if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
 			/* Check for compiled vars */
@@ -399,6 +441,10 @@ static zval* fetch_zval_from_symbol_table(HashTable *ht, char* name, int name_le
 			}
 			break;
 		case XF_ST_OBJ_PROPERTY:
+#if PHP_VERSION_ID <= 50400
+continue_from_static_root:
+#endif
+
 			/* First we try a public property */
 			element = prepare_search_key(name, &element_length, "", 0);
 			if (ht && zend_hash_find(ht, element, element_length + 1, (void **) &retval_pp) == SUCCESS) {
@@ -444,7 +490,9 @@ static zval* fetch_zval_from_symbol_table(HashTable *ht, char* name, int name_le
 			break;
 	}
 cleanup:
-	free(element);
+	if (element) {
+		free(element);
+	}
 	return retval_p;
 }
 
@@ -458,6 +506,7 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 	int        type = XF_ST_ROOT;
 	zval      *retval = NULL;
 	char      *current_classname = NULL;
+	zend_class_entry *current_ce = NULL;
 	int        cc_length = 0;
 	char       quotechar = 0;
 
@@ -483,11 +532,12 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 					if (*p[0] == '[') {
 						keyword_end = *p;
 						if (keyword) {
-							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
+							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length, current_ce TSRMLS_CC);
 							if (current_classname) {
 								efree(current_classname);
 							}
 							current_classname = NULL;
+							current_ce = NULL;
 							if (retval) {
 								st = fetch_ht_from_zval(retval TSRMLS_CC);
 							}
@@ -497,13 +547,14 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 					} else if (*p[0] == '-') {
 						keyword_end = *p;
 						if (keyword) {
-							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
+							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length, current_ce TSRMLS_CC);
 							if (current_classname) {
 								efree(current_classname);
 							}
 							current_classname = NULL;
+							current_ce = NULL;
 							if (retval) {
-								current_classname = fetch_classname_from_zval(retval, &cc_length TSRMLS_CC);
+								current_classname = fetch_classname_from_zval(retval, &cc_length, &current_ce TSRMLS_CC);
 								st = fetch_ht_from_zval(retval TSRMLS_CC);
 							}
 							keyword = NULL;
@@ -513,19 +564,19 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 					} else if (*p[0] == ':') {
 						keyword_end = *p;
 						if (keyword) {
-							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
+							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length, current_ce TSRMLS_CC);
 							if (current_classname) {
 								efree(current_classname);
 							}
 							current_classname = NULL;
 							if (retval) {
-								current_classname = fetch_classname_from_zval(retval, &cc_length TSRMLS_CC);
-								st = CE_STATIC_MEMBERS(Z_OBJCE_P(retval));
+								current_classname = fetch_classname_from_zval(retval, &cc_length, &current_ce TSRMLS_CC);
+								st = NULL;
 							}
 							keyword = NULL;
 						}
 						state = 8;
-						type = XF_ST_OBJ_PROPERTY;
+						type = XF_ST_STATIC_PROPERTY;
 					}
 					break;
 				case 2:
@@ -560,13 +611,13 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 						quotechar = 0;
 						state = 5;
 						keyword_end = *p;
-						retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
+						retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length, current_ce TSRMLS_CC);
 						if (current_classname) {
 							efree(current_classname);
 						}
 						current_classname = NULL;
 						if (retval) {
-							current_classname = fetch_classname_from_zval(retval, &cc_length TSRMLS_CC);
+							current_classname = fetch_classname_from_zval(retval, &cc_length, &current_ce TSRMLS_CC);
 							st = fetch_ht_from_zval(retval TSRMLS_CC);
 						}
 						keyword = NULL;
@@ -581,13 +632,13 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 					if (*p[0] == ']') {
 						state = 1;
 						keyword_end = *p;
-						retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
+						retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length, current_ce TSRMLS_CC);
 						if (current_classname) {
 							efree(current_classname);
 						}
 						current_classname = NULL;
 						if (retval) {
-							current_classname = fetch_classname_from_zval(retval, &cc_length TSRMLS_CC);
+							current_classname = fetch_classname_from_zval(retval, &cc_length, &current_ce TSRMLS_CC);
 							st = fetch_ht_from_zval(retval TSRMLS_CC);
 						}
 						keyword = NULL;
@@ -600,13 +651,15 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 
 						if (strncmp(keyword, "::", 2) == 0) { /* static class properties */
 							zend_class_entry *ce = zend_fetch_class(XG(active_fse)->function.class, strlen(XG(active_fse)->function.class), ZEND_FETCH_CLASS_SELF TSRMLS_CC);
-							st = CE_STATIC_MEMBERS(ce);
+
+							st = NULL;
 
 							current_classname = estrdup(ce->name);
 							cc_length = strlen(ce->name);
+							current_ce = ce;
 							keyword = *p + 1;
 
-							type = XF_ST_OBJ_PROPERTY;
+							type = XF_ST_STATIC_ROOT;
 						} else {
 							keyword = NULL;
 						}
@@ -617,10 +670,7 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 		}
 	} while (found < 0);
 	if (keyword != NULL) {
-		retval = fetch_zval_from_symbol_table(st, keyword, *p - keyword, type, current_classname, cc_length TSRMLS_CC);
-		if (retval) {
-			st = fetch_ht_from_zval(retval TSRMLS_CC);
-		}
+		retval = fetch_zval_from_symbol_table(st, keyword, *p - keyword, type, current_classname, cc_length, current_ce TSRMLS_CC);
 	}
 	if (current_classname) {
 		efree(current_classname);
@@ -748,7 +798,7 @@ static int check_evaled_code(function_stack_entry *fse, char **filename, int *li
 	filename_to_use = use_fse ? fse->filename : *filename;
 
 	end_marker = filename_to_use + strlen(filename_to_use) - strlen("eval()'d code");
-	if (strcmp("eval()'d code", end_marker) == 0) {
+	if (end_marker >= filename_to_use && strcmp("eval()'d code", end_marker) == 0) {
 		if (xdebug_hash_find(XG(context).eval_id_lookup, filename_to_use, strlen(filename_to_use), (void *) &ei)) {
 			*filename = xdebug_sprintf("dbgp://%lu", ei->id);
 		}
@@ -783,7 +833,7 @@ static xdebug_xml_node* return_stackframe(int nr TSRMLS_DC)
 		}
 		xdebug_xml_add_attribute_ex(tmp, "lineno",   xdebug_sprintf("%lu", fse_prev->lineno TSRMLS_CC), 0, 1);
 	} else {
-		tmp_filename = zend_get_executed_filename(TSRMLS_C);
+		tmp_filename = (char *) zend_get_executed_filename(TSRMLS_C);
 		tmp_lineno = zend_get_executed_lineno(TSRMLS_C);
 		if (check_evaled_code(fse, &tmp_filename, &tmp_lineno, 0 TSRMLS_CC)) {
 			xdebug_xml_add_attribute_ex(tmp, "type", xdstrdup("eval"), 0, 1);
@@ -1167,7 +1217,16 @@ DBGP_FUNC(breakpoint_set)
 				brk_info->file_len = strlen(brk_info->file);
 			}
 		} else {
+			char realpath_file[MAXPATHLEN];
+
 			brk_info->file = xdebug_path_from_url(CMD_OPTION('f') TSRMLS_CC);
+
+			/* Now we do some real path checks to resolve symlinks. */
+			if (VCWD_REALPATH(brk_info->file, realpath_file)) {
+				xdfree(brk_info->file);
+				brk_info->file = xdstrdup(realpath_file);
+			}
+
 			brk_info->file_len = strlen(brk_info->file);
 		}
 
@@ -1242,6 +1301,20 @@ static int xdebug_do_eval(char *eval_string, zval *ret_zval TSRMLS_DC)
 	zend_op_array     *original_active_op_array = EG(active_op_array);
 	zend_execute_data *original_execute_data = EG(current_execute_data);
 	int                original_no_extensions = EG(no_extensions);
+#if PHP_VERSION_ID < 50200
+	zend_bool          original_bailout_set = EG(bailout_set);
+	jmp_buf            original_bailout;
+#else
+	jmp_buf           *original_bailout = EG(bailout);
+#endif
+#if PHP_VERSION_ID >= 50300
+	void             **original_argument_stack_top = EG(argument_stack)->top;
+	void             **original_argument_stack_end = EG(argument_stack)->end;
+#endif
+
+#if PHP_VERSION_ID < 50200
+	memcpy(&original_bailout, &EG(bailout), sizeof(jmp_buf));
+#endif
 
 	/* Remember error reporting level */
 	old_error_reporting = EG(error_reporting);
@@ -1263,6 +1336,16 @@ static int xdebug_do_eval(char *eval_string, zval *ret_zval TSRMLS_DC)
 	EG(active_op_array) = original_active_op_array;
 	EG(current_execute_data) = original_execute_data;
 	EG(no_extensions) = original_no_extensions;
+#if PHP_VERSION_ID < 50200
+	EG(bailout_set) = original_bailout_set;
+	memcpy(&EG(bailout), &original_bailout, sizeof(jmp_buf));
+#else
+	EG(bailout) = original_bailout;
+#endif
+#if PHP_VERSION_ID >= 50300
+	EG(argument_stack)->top = original_argument_stack_top;
+	EG(argument_stack)->end = original_argument_stack_end;
+#endif
 
 	return res;
 }
@@ -1323,41 +1406,6 @@ static int xdebug_send_stream(const char *name, const char *str, uint str_length
 	return 0;
 }
 
-static int xdebug_header_write(const char *str, uint str_length TSRMLS_DC)
-{
-	/* nesting_level is zero when final output is sent to sapi */
-	if (OG(ob_nesting_level) < 1) {
-		zend_unset_timeout(TSRMLS_C);
-		if (XG(stdout_redirected) != 0) {
-			xdebug_send_stream("stdout", str, str_length TSRMLS_CC);
-		}
-#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
-		zend_set_timeout(EG(timeout_seconds), 0);
-#else
-		zend_set_timeout(EG(timeout_seconds));
-#endif
-	}
-	return XG(stdio).php_header_write(str, str_length TSRMLS_CC);
-}
-
-static int xdebug_body_write(const char *str, uint str_length TSRMLS_DC)
-{
-	/* nesting_level is zero when final output is sent to sapi. We also dont
-	 * want to write if headers are not sent yet, the output layer will handle
-	 * this correctly later. */
-	if (OG(ob_nesting_level) < 1 && SG(headers_sent)) {
-		zend_unset_timeout(TSRMLS_C);
-		if (XG(stdout_redirected) != 0) {
-			xdebug_send_stream("stdout", str, str_length TSRMLS_CC);
-		}
-#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
-		zend_set_timeout(EG(timeout_seconds), 0);
-#else
-		zend_set_timeout(EG(timeout_seconds));
-#endif
-	}
-	return XG(stdio).php_body_write(str, str_length TSRMLS_CC);
-}
 
 DBGP_FUNC(stderr)
 {
@@ -1374,32 +1422,12 @@ DBGP_FUNC(stdout)
 	}
 
 	mode = strtol(CMD_OPTION('c'), NULL, 10);
-
-	if (mode == 0 && XG(stdout_redirected) != 0) {
-		if (XG(stdio).php_body_write != NULL && OG(php_body_write)) {
-			OG(php_body_write) = XG(stdio).php_body_write;
-			OG(php_header_write) = XG(stdio).php_header_write;
-			
-			XG(stdio).php_body_write = NULL;
-			XG(stdio).php_header_write = NULL;
-			success = "1";
-		}
-	} else if (mode != 0 && XG(stdout_redirected) == 0) {
-		if (XG(stdio).php_body_write == NULL && OG(php_body_write)) {
-			XG(stdio).php_body_write = OG(php_body_write);
-			OG(php_body_write) = xdebug_body_write;
-			XG(stdio).php_header_write = OG(php_header_write);
-			OG(php_header_write) = xdebug_header_write;
-			success = "1";
-		}
-	}
-
-	XG(stdout_redirected) = mode;
+	XG(stdout_mode) = mode;
+	success = "1";
 
 	xdebug_xml_add_attribute_ex(*retval, "success", xdstrdup(success), 0, 1);
 }
 
-
 DBGP_FUNC(stop)
 {
 	XG(status) = DBGP_STATUS_STOPPED;
@@ -1456,9 +1484,7 @@ DBGP_FUNC(detach)
 	xdebug_xml_add_attribute(*retval, "reason", xdebug_dbgp_reason_strings[XG(reason)]);
 	XG(context).handler->remote_deinit(&(XG(context)));
 	XG(remote_enabled) = 0;
-	XG(stdout_redirected) = 0;
-	XG(stderr_redirected) = 0;
-	XG(stdin_redirected) = 0;
+	XG(stdout_mode) = 0;
 }
 
 
@@ -1930,7 +1956,6 @@ DBGP_FUNC(property_value)
 static void attach_used_var_with_contents(void *xml, xdebug_hash_element* he, void *options)
 {
 	char               *name = (char*) he->ptr;
-	char               *full_name;
 	xdebug_xml_node    *node = (xdebug_xml_node *) xml;
 	xdebug_xml_node    *contents;
 	TSRMLS_FETCH();
@@ -1943,6 +1968,41 @@ static void attach_used_var_with_contents(void *xml, xdebug_hash_element* he, vo
 	}
 }
 
+static int xdebug_add_filtered_symboltable_var(zval *symbol XDEBUG_ZEND_HASH_APPLY_TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
+{
+	xdebug_hash *tmp_hash;
+
+	tmp_hash = va_arg(args, xdebug_hash *);
+
+	if (strcmp("argc", hash_key->arKey) == 0) { return 0; }
+	if (strcmp("argv", hash_key->arKey) == 0) { return 0; }
+	if (hash_key->arKey[0] == '_') {
+		if (strcmp("_COOKIE", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("_ENV", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("_FILES", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("_GET", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("_POST", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("_REQUEST", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("_SERVER", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("_SESSION", hash_key->arKey) == 0) { return 0; }
+	}
+	if (hash_key->arKey[0] == 'H') {
+		if (strcmp("HTTP_COOKIE_VARS", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("HTTP_ENV_VARS", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("HTTP_GET_VARS", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("HTTP_POST_VARS", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("HTTP_POST_FILES", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("HTTP_RAW_POST_DATA", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("HTTP_SERVER_VARS", hash_key->arKey) == 0) { return 0; }
+		if (strcmp("HTTP_SESSION_VARS", hash_key->arKey) == 0) { return 0; }
+	}
+	if (strcmp("GLOBALS", hash_key->arKey) == 0) { return 0; }
+
+	xdebug_hash_add(tmp_hash, hash_key->arKey, strlen(hash_key->arKey), hash_key->arKey);	
+
+	return 0;
+}
+
 static int attach_context_vars(xdebug_xml_node *node, xdebug_var_export_options *options, long context_id, long depth, void (*func)(void *, xdebug_hash_element*, void*) TSRMLS_DC)
 {
 	function_stack_entry *fse;
@@ -1953,6 +2013,7 @@ static int attach_context_vars(xdebug_xml_node *node, xdebug_var_export_options
 	if (context_id == 1) {
 		/* add super globals */
 		XG(active_symbol_table) = &EG(symbol_table);
+		XG(active_execute_data) = NULL;
 		add_variable_node(node, "_COOKIE", sizeof("_COOKIE"), 1, 1, 0, options TSRMLS_CC);
 		add_variable_node(node, "_ENV", sizeof("_ENV"), 1, 1, 0, options TSRMLS_CC);
 		add_variable_node(node, "_FILES", sizeof("_FILES"), 1, 1, 0, options TSRMLS_CC);
@@ -1986,10 +2047,20 @@ static int attach_context_vars(xdebug_xml_node *node, xdebug_var_export_options
 		/* Only show vars when they are scanned */
 		if (fse->used_vars) {
 			xdebug_hash *tmp_hash;
+
+			/* Get a hash from all the used vars (which can have duplicates) */
 			tmp_hash = xdebug_used_var_hash_from_llist(fse->used_vars);
+
+			/* Check for dynamically defined variables, but make sure we don't already
+			 * have them. Also blacklist superglobals and argv/argc */
+			if (XG(active_symbol_table)) {
+				zend_hash_apply_with_arguments(XG(active_symbol_table) XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) xdebug_add_filtered_symboltable_var, 1, tmp_hash);
+			}
+
+			/* Add all the found variables to the node */
 			xdebug_hash_apply_with_argument(tmp_hash, (void *) node, func, (void *) options);
 
-			/* zend engine 2 does not give us $this, eval so we can get it */
+			/* Zend engine 2 does not give us $this, eval so we can get it */
 			if (!xdebug_hash_find(tmp_hash, "this", 4, (void *) &var_name)) {
 				add_variable_node(node, "this", sizeof("this"), 1, 1, 0, options TSRMLS_CC);
 			}
@@ -2120,7 +2191,7 @@ DBGP_FUNC(xcmd_get_executable_lines)
 	}
 
 	lines = xdebug_xml_node_init("xdebug:lines");
-	for (i = 0; i < fse->op_array->size; i++ ) {
+	for (i = 0; i < fse->op_array->last; i++ ) {
 		if (fse->op_array->opcodes[i].opcode == ZEND_EXT_STMT ) {
 			line = xdebug_xml_node_init("xdebug:line");
 			xdebug_xml_add_attribute_ex(line, "lineno", xdebug_sprintf("%lu", fse->op_array->opcodes[i].lineno), 0, 1);
@@ -2421,24 +2492,6 @@ int xdebug_dbgp_init(xdebug_con *context, int mode)
 	XG(lastcmd) = NULL;
 	XG(lasttransid) = NULL;
 
-	XG(stdout_redirected) = 0;
-	XG(stderr_redirected) = 0;
-	XG(stdin_redirected) = 0;
-	XG(stdio).php_body_write = NULL;
-	XG(stdio).php_header_write = NULL;
-
-	/* initialize remote log file */
-	XG(remote_log_file) = NULL;
-	if (XG(remote_log) && strlen(XG(remote_log))) {
-		XG(remote_log_file) = xdebug_fopen(XG(remote_log), "a", NULL, NULL);
-	}
-	if (XG(remote_log_file)) {
-		char *timestr = xdebug_get_time();
-		fprintf(XG(remote_log_file), "Log opened at %s\n", timestr);
-		fflush(XG(remote_log_file));
-		xdfree(timestr);
-	}
-
 	response = xdebug_xml_node_init("init");
 	xdebug_xml_add_attribute(response, "xmlns", "urn:debugger_protocol_v1");
 	xdebug_xml_add_attribute(response, "xmlns:xdebug", "http://xdebug.org/dbgp/xdebug");
@@ -2536,13 +2589,6 @@ int xdebug_dbgp_deinit(xdebug_con *context)
 	
 		xdebug_dbgp_cmdloop(context, 0 TSRMLS_CC);
 	}
-	if (XG(stdio).php_body_write != NULL && OG(php_body_write)) {
-		OG(php_body_write) = XG(stdio).php_body_write;
-		OG(php_header_write) = XG(stdio).php_header_write;
-		
-		XG(stdio).php_body_write = NULL;
-		XG(stdio).php_header_write = NULL;
-	}
 
 	if (XG(remote_enabled)) {
 		options = (xdebug_var_export_options*) context->options;
@@ -2556,14 +2602,7 @@ int xdebug_dbgp_deinit(xdebug_con *context)
 		xdfree(context->buffer);
 	}
 
-	if (XG(remote_log_file)) {
-		char *timestr = xdebug_get_time();
-		fprintf(XG(remote_log_file), "Log closed at %s\n\n", timestr);
-		fflush(XG(remote_log_file));
-		xdfree(timestr);
-		fclose(XG(remote_log_file));
-		XG(remote_log_file) = NULL;
-	}
+	xdebug_close_log(TSRMLS_C);
 	XG(remote_enabled) = 0;
 	return 1;
 }
@@ -2692,6 +2731,18 @@ int xdebug_dbgp_breakpoint(xdebug_con *context, xdebug_llist *stack, char *file,
 	return 1;
 }
 
+int xdebug_dbgp_stream_output(const char *string, unsigned int length TSRMLS_DC)
+{
+	if ((XG(stdout_mode) == 1 || XG(stdout_mode) == 2) && length) {
+		xdebug_send_stream("stdout", string, length TSRMLS_CC);
+	}
+
+	if (XG(stdout_mode) == 0 || XG(stdout_mode) == 1) {
+		return 0;
+	}
+	return -1;
+}
+
 static char *create_eval_key_file(char *filename, int lineno)
 {
 	return xdebug_sprintf("%s(%d) : eval()'d code", filename, lineno);
