@@ -30,7 +30,7 @@
 
 ZEND_EXTERN_MODULE_GLOBALS(xdebug)
 
-static char* text_formats[10] = {
+static char* text_formats[11] = {
 	"\n",
 	"%s: %s in %s on line %d\n",
 	"\nCall Stack:\n",
@@ -44,11 +44,30 @@ static char* text_formats[10] = {
 	"\n\nVariables in local scope (#%d):\n",
 	"\n",
 	"  $%s = %s\n",
-	"  $%s = *uninitialized*\n"
+	"  $%s = *uninitialized*\n",
+	"SCREAM:  Error suppression ignored for\n"
 };
 
-static char* html_formats[12] = {
-	"<br />\n<font size='1'><table class='xdebug-error' dir='ltr' border='1' cellspacing='0' cellpadding='1'>\n",
+static char* ansi_formats[11] = {
+	"\n",
+	"[1m[31m%s[0m: %s[22m in [31m%s[0m on line [32m%d[0m[22m\n",
+	"\n[1mCall Stack:[22m\n",
+#if HAVE_PHP_MEMORY_USAGE
+	"%10.4f %10ld %3d. %s(",
+#else
+	"%10.4f %3d. %s(",
+#endif
+	"'%s'",
+	") %s:%d\n",
+	"\n\nVariables in local scope (#%d):\n",
+	"\n",
+	"  $%s = %s\n",
+	"  $%s = *uninitialized*\n",
+	"[1m[31mSCREAM[0m:  Error suppression ignored for\n"
+};
+
+static char* html_formats[13] = {
+	"<br />\n<font size='1'><table class='xdebug-error xe-%s%s' dir='ltr' border='1' cellspacing='0' cellpadding='1'>\n",
 	"<tr><th align='left' bgcolor='#f57900' colspan=\"5\"><span style='background-color: #cc0000; color: #fce94f; font-size: x-large;'>( ! )</span> %s: %s in %s on line <i>%d</i></th></tr>\n",
 #if HAVE_PHP_MEMORY_USAGE
 	"<tr><th align='left' bgcolor='#e9b96e' colspan='5'>Call Stack</th></tr>\n<tr><th align='center' bgcolor='#eeeeec'>#</th><th align='left' bgcolor='#eeeeec'>Time</th><th align='left' bgcolor='#eeeeec'>Memory</th><th align='left' bgcolor='#eeeeec'>Function</th><th align='left' bgcolor='#eeeeec'>Location</th></tr>\n",
@@ -68,9 +87,22 @@ static char* html_formats[12] = {
 	"<tr><td colspan='2' align='right' bgcolor='#eeeeec' valign='top'><pre>$%s&nbsp;=</pre></td><td colspan='3' bgcolor='#eeeeec'>%s</td></tr>\n",
 	"<tr><td colspan='2' align='right' bgcolor='#eeeeec' valign='top'><pre>$%s&nbsp;=</pre></td><td colspan='3' bgcolor='#eeeeec' valign='top'><i>Undefined</i></td></tr>\n",
 	" )</td><td title='%s' bgcolor='#eeeeec'><a style='color: black' href='%s'>..%s<b>:</b>%d</a></td></tr>\n",
-	"<tr><th align='left' bgcolor='#f57900' colspan=\"5\"><span style='background-color: #cc0000; color: #fce94f; font-size: x-large;'>( ! )</span> %s: %s in <a style='color: black' href='%s'>%s</a> on line <i>%d</i></th></tr>\n"
+	"<tr><th align='left' bgcolor='#f57900' colspan=\"5\"><span style='background-color: #cc0000; color: #fce94f; font-size: x-large;'>( ! )</span> %s: %s in <a style='color: black' href='%s'>%s</a> on line <i>%d</i></th></tr>\n",
+	"<tr><th align='left' bgcolor='#f57900' colspan=\"5\"><span style='background-color: #cc0000; color: #fce94f; font-size: x-large;'>( ! )</span> SCREAM: Error suppression ignored for</th></tr>\n"
 };
 
+static char** select_formats(int html TSRMLS_DC) {
+	if (html) {
+		return html_formats;
+	} 
+	else if ((XG(cli_color) == 1 && xdebug_is_output_tty(TSRMLS_C)) || (XG(cli_color) == 2)) {
+		return ansi_formats;
+	}
+	else {
+		return text_formats;
+	}
+}
+
 static void dump_used_var_with_contents(void *htmlq, xdebug_hash_element* he, void *argument)
 {
 	int        html = *(int *)htmlq;
@@ -103,11 +135,7 @@ static void dump_used_var_with_contents(void *htmlq, xdebug_hash_element* he, vo
 	zvar = xdebug_get_php_symbol(name, strlen(name) + 1);
 	XG(active_symbol_table) = tmp_ht;
 
-	if (html) {
-		formats = html_formats;
-	} else {
-		formats = text_formats;
-	}
+	formats = select_formats(PG(html_errors) TSRMLS_CC);
 
 	if (!zvar) {
 		xdebug_str_add(str, xdebug_sprintf(formats[9], name), 1);
@@ -217,16 +245,26 @@ static int create_file_link(char **filename, const char *error_filename, int err
 	return fname.l;
 }
 
-static void xdebug_append_error_head(xdebug_str *str, int html TSRMLS_DC)
+void xdebug_append_error_head(xdebug_str *str, int html, char *error_type_str TSRMLS_DC)
 {
-	char **formats = html ? html_formats : text_formats;
- 
-	xdebug_str_add(str, formats[0], 0);
+	char **formats = select_formats(html TSRMLS_CC);
+
+	if (html) {
+		xdebug_str_add(str, xdebug_sprintf(formats[0], error_type_str, XG(in_at) ? " xe-scream" : ""), 1);
+		if (XG(in_at)) {
+			xdebug_str_add(str, formats[12], 0);
+		}
+	} else {
+		xdebug_str_add(str, formats[0], 0);
+		if (XG(in_at)) {
+			xdebug_str_add(str, formats[10], 0);
+		}
+	}
 }
 
 void xdebug_append_error_description(xdebug_str *str, int html, const char *error_type_str, char *buffer, const char *error_filename, const int error_lineno TSRMLS_DC)
 {
-	char **formats = html ? html_formats : text_formats;
+	char **formats = select_formats(html TSRMLS_CC);
 	char *escaped;
 #if PHP_VERSION_ID >= 50400
 	size_t newlen;
@@ -235,7 +273,7 @@ void xdebug_append_error_description(xdebug_str *str, int html, const char *erro
 #endif
 
 	if (html) {
-		escaped = php_escape_html_entities(buffer, strlen(buffer), &newlen, 0, 0, NULL TSRMLS_CC);
+		escaped = php_escape_html_entities((unsigned char *) buffer, strlen(buffer), &newlen, 0, 0, NULL TSRMLS_CC);
 	} else {
 		escaped = estrdup(buffer);
 	}
@@ -258,7 +296,7 @@ void xdebug_append_printable_stack(xdebug_str *str, int html TSRMLS_DC)
 	xdebug_llist_element *le;
 	function_stack_entry *i;
 	int    len;
-	char **formats = html ? html_formats : text_formats;
+	char **formats = select_formats(html TSRMLS_CC);
 
 	if (XG(stack) && XG(stack)->size) {
 		i = XDEBUG_LLIST_VALP(XDEBUG_LLIST_HEAD(XG(stack)));
@@ -313,13 +351,13 @@ void xdebug_append_printable_stack(xdebug_str *str, int html TSRMLS_DC)
 						tmp_fancy_value = xdebug_xmlize(tmp_value, strlen(tmp_value), &newlen);
 						tmp_fancy_synop_value = xdebug_get_zval_synopsis_fancy("", i->var[j].addr, &len, 0, NULL TSRMLS_CC);
 						switch (XG(collect_params)) {
-							case 1: // synopsis
+							case 1: /* synopsis */
 								xdebug_str_add(str, xdebug_sprintf("<span>%s</span>", tmp_fancy_synop_value), 1);
 								break;
-							case 2: // synopsis + full in tooltip
+							case 2: /* synopsis + full in tooltip */
 								xdebug_str_add(str, xdebug_sprintf("<span title='%s'>%s</span>", tmp_fancy_value, tmp_fancy_synop_value), 1);
 								break;
-							case 3: // full
+							case 3: /* full */
 							default:
 								xdebug_str_add(str, xdebug_sprintf("<span>%s</span>", tmp_fancy_value), 1);
 								break;
@@ -329,7 +367,7 @@ void xdebug_append_printable_stack(xdebug_str *str, int html TSRMLS_DC)
 						xdfree(tmp_fancy_synop_value);
 					} else {
 						switch (XG(collect_params)) {
-							case 1: // synopsis
+							case 1: /* synopsis */
 							case 2:
 								tmp_value = xdebug_get_zval_synopsis(i->var[j].addr, 0, NULL);
 								break;
@@ -401,57 +439,88 @@ void xdebug_append_printable_stack(xdebug_str *str, int html TSRMLS_DC)
 	}
 }
 
-static void xdebug_append_error_footer(xdebug_str *str, int html)
+void xdebug_append_error_footer(xdebug_str *str, int html TSRMLS_DC)
 {
-	char **formats = html ? html_formats : text_formats;
+	char **formats = select_formats(html TSRMLS_CC);
 
 	xdebug_str_add(str, formats[7], 0);
 }
 
-static char *get_printable_stack(int html, const char *error_type_str, char *buffer, const char *error_filename, const int error_lineno TSRMLS_DC)
+static char *get_printable_stack(int html, int error_type, char *buffer, const char *error_filename, const int error_lineno TSRMLS_DC)
 {
 	char *prepend_string;
 	char *append_string;
+	char *error_type_str = xdebug_error_type(error_type);
+	char *error_type_str_simple = xdebug_error_type_simple(error_type);
 	xdebug_str str = {0, 0, NULL};
 
 	prepend_string = INI_STR("error_prepend_string");
 	append_string = INI_STR("error_append_string");
 
 	xdebug_str_add(&str, prepend_string ? prepend_string : "", 0);
-	xdebug_append_error_head(&str, html TSRMLS_CC);
+	xdebug_append_error_head(&str, html, error_type_str_simple TSRMLS_CC);
 	xdebug_append_error_description(&str, html, error_type_str, buffer, error_filename, error_lineno TSRMLS_CC);
 	xdebug_append_printable_stack(&str, html TSRMLS_CC);
-	xdebug_append_error_footer(&str, html);
+	xdebug_append_error_footer(&str, html TSRMLS_CC);
 	xdebug_str_add(&str, append_string ? append_string : "", 0);
 
+	xdfree(error_type_str);
+	xdfree(error_type_str_simple);
+
 	return str.d;
 }
 
+#define XDEBUG_LOG_PRINT(fs, string, ...) if (fs) { fprintf(fs, string, ## __VA_ARGS__); }
+
+void xdebug_init_debugger(TSRMLS_D)
+{
+	xdebug_open_log(TSRMLS_C);
+	if (XG(remote_connect_back)) {
+		zval **remote_addr = NULL;
+		XDEBUG_LOG_PRINT(XG(remote_log_file), "I: Checking remote connect back address.\n");
+		if (zend_hash_find(Z_ARRVAL_P(PG(http_globals)[TRACK_VARS_SERVER]), "X_HTTP_FORWARDED_FOR", 21, (void**)&remote_addr) == FAILURE) {
+			zend_hash_find(Z_ARRVAL_P(PG(http_globals)[TRACK_VARS_SERVER]), "REMOTE_ADDR", 12, (void**)&remote_addr);
+		}
+		if (remote_addr) {
+			XDEBUG_LOG_PRINT(XG(remote_log_file), "I: Remote address found, connecting to %s:%ld.\n", Z_STRVAL_PP(remote_addr), XG(remote_port));
+			XG(context).socket = xdebug_create_socket(Z_STRVAL_PP(remote_addr), XG(remote_port));
+		} else {
+			XDEBUG_LOG_PRINT(XG(remote_log_file), "W: Remote address not found, connecting to configured address/port: %s:%ld. :-|\n", XG(remote_host), XG(remote_port));
+			XG(context).socket = xdebug_create_socket(XG(remote_host), XG(remote_port));
+		}
+	} else {
+		XDEBUG_LOG_PRINT(XG(remote_log_file), "I: Connecting to configured address/port: %s:%ld.\n", XG(remote_host), XG(remote_port));
+		XG(context).socket = xdebug_create_socket(XG(remote_host), XG(remote_port));
+	}
+	if (XG(context).socket >= 0) {
+		XDEBUG_LOG_PRINT(XG(remote_log_file), "I: Connected to client. :-)\n");
+		XG(remote_enabled) = 0;
+
+		/* Get handler from mode */
+		XG(context).handler = xdebug_handler_get(XG(remote_handler));
+		if (!XG(context).handler) {
+			zend_error(E_WARNING, "The remote debug handler '%s' is not supported.", XG(remote_handler));
+			XDEBUG_LOG_PRINT(XG(remote_log_file), "E: The remote debug handler '%s' is not supported. :-(\n", XG(remote_handler));
+		} else if (!XG(context).handler->remote_init(&(XG(context)), XDEBUG_REQ)) {
+			/* The request could not be started, ignore it then */
+			XDEBUG_LOG_PRINT(XG(remote_log_file), "E: The debug session could not be started. :-(\n");
+		} else {
+			/* All is well, turn off script time outs */
+			zend_alter_ini_entry("max_execution_time", sizeof("max_execution_time"), "0", strlen("0"), PHP_INI_SYSTEM, PHP_INI_STAGE_ACTIVATE);
+			XG(remote_enabled) = 1;
+		}
+	} else {
+		XDEBUG_LOG_PRINT(XG(remote_log_file), "E: Could not connect to client. :-(\n");
+	}
+	if (!XG(remote_enabled)) {
+		xdebug_close_log(TSRMLS_C);
+	}
+}
+
 void xdebug_do_jit(TSRMLS_D)
 {
 	if (!XG(remote_enabled) && XG(remote_enable) && (XG(remote_mode) == XDEBUG_JIT)) {
-		if (XG(remote_connect_back)) {
-			zval **remote_addr = NULL;
-			zend_hash_find(Z_ARRVAL_P(PG(http_globals)[TRACK_VARS_SERVER]), "REMOTE_ADDR", 12, (void**)&remote_addr);
-			XG(context).socket = xdebug_create_socket(Z_STRVAL_PP(remote_addr), XG(remote_port));
-		} else {
-			XG(context).socket = xdebug_create_socket(XG(remote_host), XG(remote_port));
-		}
-		if (XG(context).socket >= 0) {
-			XG(remote_enabled) = 0;
-
-			/* Get handler from mode */
-			XG(context).handler = xdebug_handler_get(XG(remote_handler));
-			if (!XG(context).handler) {
-				zend_error(E_WARNING, "The remote debug handler '%s' is not supported.", XG(remote_handler));
-			} else if (!XG(context).handler->remote_init(&(XG(context)), XDEBUG_JIT)) {
-				/* The request could not be started, ignore it then */
-			} else {
-				/* All is well, turn off script time outs */
-				zend_alter_ini_entry("max_execution_time", sizeof("max_execution_time"), "0", strlen("0"), PHP_INI_SYSTEM, PHP_INI_STAGE_ACTIVATE);
-				XG(remote_enabled) = 1;
-			}
-		}
+		xdebug_init_debugger(TSRMLS_C);
 	}
 }
 
@@ -560,24 +629,27 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 				strncpy(tmp_buf, buffer, p - buffer );
 
 				/* Append error */
-				xdebug_append_error_head(&str, PG(html_errors) TSRMLS_CC);
+				xdebug_append_error_head(&str, PG(html_errors), "uncaught-exception" TSRMLS_CC);
 				xdebug_append_error_description(&str, PG(html_errors), error_type_str, tmp_buf, error_filename, error_lineno TSRMLS_CC);
 				xdebug_append_printable_stack(&str, PG(html_errors) TSRMLS_CC);
 				xdebug_str_add(&str, XG(last_exception_trace), 0);
-				xdebug_append_error_footer(&str, PG(html_errors));
+				xdebug_append_error_footer(&str, PG(html_errors) TSRMLS_CC);
 				php_output_error(str.d TSRMLS_CC);
 
 				xdfree(str.d);
 				free(tmp_buf);
 			} else {
-				printable_stack = get_printable_stack(PG(html_errors), error_type_str, buffer, error_filename, error_lineno TSRMLS_CC);
-				php_output_error(printable_stack TSRMLS_CC);
-				xdfree(printable_stack);
+				printable_stack = get_printable_stack(PG(html_errors), type, buffer, error_filename, error_lineno TSRMLS_CC);
+				if (XG(do_collect_errors) && (type != E_ERROR) && (type != E_COMPILE_ERROR) && (type != E_USER_ERROR)) {
+					xdebug_llist_insert_next(XG(collected_errors), XDEBUG_LLIST_TAIL(XG(collected_errors)), printable_stack);
+				} else {
+					php_output_error(printable_stack TSRMLS_CC);
+					xdfree(printable_stack);
+				}
 			}
-		}
-		if (XG(do_collect_errors)) {
+		} else if (XG(do_collect_errors)) {
 			char *printable_stack;
-			printable_stack = get_printable_stack(PG(html_errors), error_type_str, buffer, error_filename, error_lineno TSRMLS_CC);
+			printable_stack = get_printable_stack(PG(html_errors), type, buffer, error_filename, error_lineno TSRMLS_CC);
 			xdebug_llist_insert_next(XG(collected_errors), XDEBUG_LLIST_TAIL(XG(collected_errors)), printable_stack);
 		}
 	}
@@ -589,7 +661,7 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 	if (XG(remote_enabled) && XG(breakpoints_allowed)) {
 		if (xdebug_hash_find(XG(context).exception_breakpoints, error_type_str, strlen(error_type_str), (void *) &extra_brk_info)) {
 			if (xdebug_handle_hit_value(extra_brk_info)) {
-				if (!XG(context).handler->remote_breakpoint(&(XG(context)), XG(stack), error_filename, error_lineno, XDEBUG_BREAK, error_type_str, buffer)) {
+				if (!XG(context).handler->remote_breakpoint(&(XG(context)), XG(stack), (char *) error_filename, error_lineno, XDEBUG_BREAK, error_type_str, buffer)) {
 					XG(remote_enabled) = 0;
 				}
 			}
@@ -693,9 +765,9 @@ PHP_FUNCTION(xdebug_print_function_stack)
  
 	i = xdebug_get_stack_frame(0 TSRMLS_CC);
 	if (message) {
-		tmp = get_printable_stack(PG(html_errors), "Xdebug", message, i->filename, i->lineno TSRMLS_CC);
+		tmp = get_printable_stack(PG(html_errors), 0, message, i->filename, i->lineno TSRMLS_CC);
 	} else {
-		tmp = get_printable_stack(PG(html_errors), "Xdebug", "user triggered", i->filename, i->lineno TSRMLS_CC);
+		tmp = get_printable_stack(PG(html_errors), 0, "user triggered", i->filename, i->lineno TSRMLS_CC);
 	}
 	php_printf("%s", tmp);
 	xdfree(tmp);
@@ -710,7 +782,7 @@ PHP_FUNCTION(xdebug_get_formatted_function_stack)
 	char *tmp;
 
 	i = xdebug_get_stack_frame(0 TSRMLS_CC);
-	tmp = get_printable_stack(PG(html_errors), "Xdebug", "user triggered", i->filename, i->lineno TSRMLS_CC);
+	tmp = get_printable_stack(PG(html_errors), 0, "user triggered", i->filename, i->lineno TSRMLS_CC);
 	RETVAL_STRING(tmp, 1);
 	xdfree(tmp);
 }
@@ -809,9 +881,22 @@ static void xdebug_build_fname(xdebug_func *tmp, zend_execute_data *edata TSRMLS
 			} else {
 				tmp->type = XFUNC_NORMAL;
 			}
-			tmp->function = xdstrdup(edata->function_state.function->common.function_name);
+			if (strcmp(edata->function_state.function->common.function_name, "{closure}") == 0) {
+				tmp->function = xdebug_sprintf(
+					"{closure:%s:%d-%d}",
+					edata->function_state.function->op_array.filename,
+					edata->function_state.function->op_array.line_start,
+					edata->function_state.function->op_array.line_end
+				);
+			} else {
+				tmp->function = xdstrdup(edata->function_state.function->common.function_name);
+			}
 		} else {
+#if PHP_VERSION_ID >= 50399
+			switch (edata->opline->extended_value) {
+#else
 			switch (edata->opline->op2.u.constant.value.lval) {
+#endif
 				case ZEND_EVAL:
 					tmp->type = XFUNC_EVAL;
 					break;
@@ -842,8 +927,8 @@ function_stack_entry *xdebug_add_stack_frame(zend_execute_data *zdata, zend_op_a
 	zend_op              *cur_opcode;
 	zval                **param;
 	int                   i = 0;
-	char                 *aggr_key;
-	int                   aggr_key_len;
+	char                 *aggr_key = NULL;
+	int                   aggr_key_len = 0;
 
 	tmp = xdmalloc (sizeof (function_stack_entry));
 	tmp->var           = NULL;
@@ -855,7 +940,7 @@ function_stack_entry *xdebug_add_stack_frame(zend_execute_data *zdata, zend_op_a
 	tmp->user_defined  = type;
 	tmp->filename      = NULL;
 	tmp->include_filename  = NULL;
-	tmp->profile.call_list = xdebug_llist_alloc(xdebug_profile_call_entry_dtor);
+	tmp->profile.call_list = NULL;
 	tmp->op_array      = op_array;
 	tmp->symbol_table  = NULL;
 	tmp->execute_data  = NULL;
@@ -1140,6 +1225,7 @@ PHP_FUNCTION(xdebug_get_function_stack)
 			add_assoc_string_ex(frame, "function", sizeof("function"), i->function.function, 1);
 		}
 		if (i->function.class) {
+			add_assoc_string_ex(frame, "type",     sizeof("type"),     i->function.type == XFUNC_STATIC_MEMBER ? "static" : "dynamic", 1);
 			add_assoc_string_ex(frame, "class",    sizeof("class"),    i->function.class,    1);
 		}
 		add_assoc_string_ex(frame, "file", sizeof("file"), i->filename, 1);
