@@ -70,14 +70,14 @@ char* xdebug_return_trace_assignment(function_stack_entry *i, char *varname, zva
 	if (XG(show_mem_delta)) {
 		xdebug_str_addl(&str, "        ", 8, 0);
 	}
-	for (j = 0; j < i->level; j++) {
+	for (j = 0; j <= i->level; j++) {
 		xdebug_str_addl(&str, "  ", 2, 0);
 	}
 	xdebug_str_addl(&str, "   => ", 6, 0);
 
 	xdebug_str_add(&str, varname, 0);
 
-	if (op[0] != '\0' ) { // pre/post inc/dec ops are special
+	if (op[0] != '\0' ) { /* pre/post inc/dec ops are special */
 		xdebug_str_add(&str, xdebug_sprintf(" %s ", op), 1);
 
 		tmp_value = xdebug_get_zval_value(retval, 0, NULL);
@@ -158,7 +158,7 @@ static char* return_trace_stack_frame_begin_normal(function_stack_entry* i TSRML
 			}
 
 			switch (XG(collect_params)) {
-				case 1: // synopsis
+				case 1: /* synopsis */
 				case 2:
 					tmp_value = xdebug_get_zval_synopsis(i->var[j].addr, 0, NULL);
 					break;
@@ -251,7 +251,7 @@ static char* return_trace_stack_frame_computerized(function_stack_entry* i, int
 				}
 
 				switch (XG(collect_params)) {
-					case 1: // synopsis
+					case 1: /* synopsis */
 					case 2:
 						tmp_value = xdebug_get_zval_synopsis(i->var[j].addr, 0, NULL);
 						break;
@@ -308,8 +308,7 @@ static char* return_trace_stack_frame_begin_html(function_stack_entry* i, int fn
 
 	if (i->include_filename) {
 		if (i->function.type == XFUNC_EVAL) {
-			char             *key, *joined;
-			xdebug_eval_info *ei;
+			char             *joined;
 			xdebug_arg       *parts = (xdebug_arg*) xdmalloc(sizeof(xdebug_arg));
 
 			xdebug_arg_init(parts);
@@ -443,12 +442,15 @@ void xdebug_stop_trace(TSRMLS_D)
 {
 	char   *str_time;
 	double  u_time;
+	char   *tmp;
 
 	XG(do_trace) = 0;
 	if (XG(trace_file)) {
 		if (XG(trace_format) == 0 || XG(trace_format) == 1) {
 			u_time = xdebug_get_utime();
-			fprintf(XG(trace_file), XG(trace_format) == 0 ? "%10.4f " : "\t\t\t%f\t", u_time - XG(start_time));
+			tmp = xdebug_sprintf(XG(trace_format) == 0 ? "%10.4f " : "\t\t\t%f\t", u_time - XG(start_time));
+			fprintf(XG(trace_file), tmp);
+			xdfree(tmp);
 #if HAVE_PHP_MEMORY_USAGE
 			fprintf(XG(trace_file), XG(trace_format) == 0 ? "%10zu" : "%lu", XG_MEMORY_USAGE());
 #else
