@@ -61,7 +61,7 @@ struct _xdebug_xml_node
 #define xdebug_xml_add_attribute(x,a,v)    xdebug_xml_add_attribute_ex((x), (a), (v), 0, 0);
 
 xdebug_xml_node *xdebug_xml_node_init_ex(char *tag, int free_tag);
-void xdebug_xml_add_attribute_exl(xdebug_xml_node* xml, char *attribute, int attribute_len, char *value, int varlue_len, int free_name, int free_value);
+void xdebug_xml_add_attribute_exl(xdebug_xml_node* xml, char *attribute, size_t attribute_len, char *value, size_t value_len, int free_name, int free_value);
 void xdebug_xml_add_child(xdebug_xml_node *xml, xdebug_xml_node *child);
 
 void xdebug_xml_add_text_ex(xdebug_xml_node *xml, char *text, int length, int free_text, int encode);
